import json

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.base import deep_walk
from pypanther.helpers.msft import azure_rule_context


@panther_managed
class AzureAuditMFADisabled(Rule):
    id = "Azure.Audit.MFADisabled-prototype"
    display_name = "Azure MFA Disabled"
    log_types = [LogType.AZURE_AUDIT]
    default_severity = Severity.HIGH
    default_description = "This detection looks for MFA being disabled in conditional access policy\n"
    reports = {"MITRE ATT&CK": ["TA0005:T1556", "TA0001:T1078"]}
    default_runbook = "Verify if the change was authorized and investigate the user activity. If unauthorized, re-enable MFA, revoke access.\n"
    default_reference = "https://learn.microsoft.com/en-us/entra/identity/authentication/overview-authentication"
    summary_attributes = ["properties:ServicePrincipalName", "properties:UserPrincipalName", "properties:ipAddress"]

    def get_mfa(self, policy):
        parse_one = json.loads(policy)
        mfa_get = deep_walk(parse_one, "grantControls", "builtInControls", default=[])
        mfa_standardized = [n.lower() for n in mfa_get]
        return mfa_standardized

    def rule(self, event):
        if event.get("operationName", default="") != "Update conditional access policy":
            return False
        old_value = event.deep_walk(
            "properties",
            "targetResources",
            "modifiedProperties",
            "oldValue",
            return_val="first",
            default="",
        )
        new_value = event.deep_walk(
            "properties",
            "targetResources",
            "modifiedProperties",
            "newValue",
            return_val="first",
            default="",
        )
        old_value_parsed = self.get_mfa(old_value)
        new_value_parsed = self.get_mfa(new_value)
        return "mfa" in old_value_parsed and "mfa" not in new_value_parsed

    def title(self, event):
        actor_name = event.deep_get("properties", "initiatedBy", "user", "userPrincipalName", default="<UNKNOWN ACTOR>")
        policy = event.deep_walk("properties", "targetResources", "displayName", default="")
        return f"MFA disabled by {actor_name} on the policy {policy}"

    def alert_context(self, event):
        return azure_rule_context(event)

    tests = [
        RuleTest(
            name="MFA Disabled Successful",
            expected_result=True,
            log={
                "time": "2024-11-27T03:31:26.7088498Z",
                "resourceId": "/tenants/123456789/providers/Microsoft.aadiam",
                "operationName": "Update conditional access policy",
                "operationVersion": "1.0",
                "category": "AuditLogs",
                "tenantId": "123456789",
                "resultSignature": "None",
                "durationMs": 0,
                "callerIpAddress": "1.2.3.4",
                "correlationId": "123456789",
                "Level": 4,
                "properties": {
                    "tenantId": "123456789",
                    "resultType": "",
                    "resultDescription": "",
                    "operationName": "Update conditional access policy",
                    "identity": "",
                    "tenantGeo": "NA",
                    "id": "IPCGraph_123456789",
                    "category": "Policy",
                    "correlationId": "123456789",
                    "result": "success",
                    "resultReason": None,
                    "activityDisplayName": "Update conditional access policy",
                    "activityDateTime": "2024-11-27T03:31:26.7088498+00:00",
                    "loggedByService": "Conditional Access",
                    "operationType": "Update",
                    "userAgent": None,
                    "initiatedBy": {
                        "user": {
                            "id": "123456789b",
                            "displayName": None,
                            "userPrincipalName": "Kratos@onmicrosoft.com",
                            "ipAddress": "1.2.3.4",
                            "roles": [],
                        },
                    },
                    "targetResources": [
                        {
                            "id": "123456789",
                            "displayName": "MFA",
                            "type": "Policy",
                            "modifiedProperties": [
                                {
                                    "displayName": "ConditionalAccessPolicy",
                                    "oldValue": '{"id":"123456789","displayName":"MFA","createdDateTime":"2024-11-21T16:48:48.1196443+00:00","modifiedDateTime":"2024-11-21T16:56:13.9120766+00:00","state":"enabled","conditions":{"applications":{"includeApplications":["None"],"excludeApplications":[],"includeUserActions":[],"includeAuthenticationContextClassReferences":[],"applicationFilter":null},"users":{"includeUsers":["All"],"excludeUsers":[],"includeGroups":[],"excludeGroups":[],"includeRoles":[],"excludeRoles":[]},"userRiskLevels":[],"signInRiskLevels":[],"clientAppTypes":["all"],"servicePrincipalRiskLevels":[]},"grantControls":{"operator":"OR","builtInControls":["MFA"],"customAuthenticationFactors":[],"termsOfUse":[]},"sessionControls":{"signInFrequency":{"value":90,"type":"days","authenticationType":"primaryAndSecondaryAuthentication","frequencyInterval":"timeBased","isEnabled":true}}}',
                                    "newValue": '{"id":"123456789","displayName":"MFA","createdDateTime":"2024-11-21T16:48:48.1196443+00:00","modifiedDateTime":"2024-11-27T03:31:25.4989035+00:00","state":"enabled","conditions":{"applications":{"includeApplications":["None"],"excludeApplications":[],"includeUserActions":[],"includeAuthenticationContextClassReferences":[],"applicationFilter":null},"users":{"includeUsers":["All"],"excludeUsers":[],"includeGroups":[],"excludeGroups":[],"includeRoles":[],"excludeRoles":[]},"userRiskLevels":[],"signInRiskLevels":[],"clientAppTypes":["all"],"servicePrincipalRiskLevels":[]},"sessionControls":{"signInFrequency":{"value":90,"type":"days","authenticationType":"primaryAndSecondaryAuthentication","frequencyInterval":"timeBased","isEnabled":true}}}',
                                },
                            ],
                            "administrativeUnits": [],
                        },
                    ],
                    "additionalDetails": [{"key": "Category", "value": "Conditional Access"}],
                },
            },
        ),
        RuleTest(
            name="MFA Enabled",
            expected_result=False,
            log={
                "time": "2024-11-27T03:31:26.7088498Z",
                "resourceId": "/tenants/123456789/providers/Microsoft.aadiam",
                "operationName": "Update conditional access policy",
                "operationVersion": "1.0",
                "category": "AuditLogs",
                "tenantId": "123456789",
                "resultSignature": "None",
                "durationMs": 0,
                "callerIpAddress": "1.2.3.4",
                "correlationId": "123456789",
                "Level": 4,
                "properties": {
                    "tenantId": "123456789",
                    "resultType": "",
                    "resultDescription": "",
                    "operationName": "Update conditional access policy",
                    "identity": "",
                    "tenantGeo": "NA",
                    "id": "IPCGraph_123456789",
                    "category": "Policy",
                    "correlationId": "123456789",
                    "result": "success",
                    "resultReason": None,
                    "activityDisplayName": "Update conditional access policy",
                    "activityDateTime": "2024-11-27T03:31:26.7088498+00:00",
                    "loggedByService": "Conditional Access",
                    "operationType": "Update",
                    "userAgent": None,
                    "initiatedBy": {
                        "user": {
                            "id": "123456789b",
                            "displayName": None,
                            "userPrincipalName": "Kratos@onmicrosoft.com",
                            "ipAddress": "1.2.3.4",
                            "roles": [],
                        },
                    },
                    "targetResources": [
                        {
                            "id": "123456789",
                            "displayName": "MFA",
                            "type": "Policy",
                            "modifiedProperties": [
                                {
                                    "displayName": "ConditionalAccessPolicy",
                                    "oldValue": '{"id":"123456789","displayName":"MFA","createdDateTime":"2024-11-21T16:48:48.1196443+00:00","modifiedDateTime":"2024-11-27T03:31:25.4989035+00:00","state":"enabled","conditions":{"applications":{"includeApplications":["None"],"excludeApplications":[],"includeUserActions":[],"includeAuthenticationContextClassReferences":[],"applicationFilter":null},"users":{"includeUsers":["All"],"excludeUsers":[],"includeGroups":[],"excludeGroups":[],"includeRoles":[],"excludeRoles":[]},"userRiskLevels":[],"signInRiskLevels":[],"clientAppTypes":["all"],"servicePrincipalRiskLevels":[]},"sessionControls":{"signInFrequency":{"value":90,"type":"days","authenticationType":"primaryAndSecondaryAuthentication","frequencyInterval":"timeBased","isEnabled":true}}}',
                                    "newValue": '{"id":"123456789","displayName":"MFA","createdDateTime":"2024-11-21T16:48:48.1196443+00:00","modifiedDateTime":"2024-11-21T16:56:13.9120766+00:00","state":"enabled","conditions":{"applications":{"includeApplications":["None"],"excludeApplications":[],"includeUserActions":[],"includeAuthenticationContextClassReferences":[],"applicationFilter":null},"users":{"includeUsers":["All"],"excludeUsers":[],"includeGroups":[],"excludeGroups":[],"includeRoles":[],"excludeRoles":[]},"userRiskLevels":[],"signInRiskLevels":[],"clientAppTypes":["all"],"servicePrincipalRiskLevels":[]},"grantControls":{"operator":"OR","builtInControls":["MFA"],"customAuthenticationFactors":[],"termsOfUse":[]},"sessionControls":{"signInFrequency":{"value":90,"type":"days","authenticationType":"primaryAndSecondaryAuthentication","frequencyInterval":"timeBased","isEnabled":true}}}',
                                },
                            ],
                            "administrativeUnits": [],
                        },
                    ],
                    "additionalDetails": [{"key": "Category", "value": "Conditional Access"}],
                },
            },
        ),
        RuleTest(
            name="MFA Disabled from another log",
            expected_result=False,
            log={
                "time": "2024-11-27T03:31:26.2934305Z",
                "resourceId": "/tenants/123456/providers/Microsoft.aadiam",
                "operationName": "Update policy",
                "operationVersion": "1.0",
                "category": "AuditLogs",
                "tenantId": "123456",
                "resultSignature": "None",
                "durationMs": 0,
                "callerIpAddress": "1.2.3.4",
                "correlationId": "123456",
                "Level": 4,
                "properties": {
                    "tenantId": "123456",
                    "resultType": "",
                    "resultDescription": "",
                    "operationName": "Update policy",
                    "identity": "",
                    "tenantGeo": "NA",
                    "id": "Directory_123145",
                    "category": "Policy",
                    "correlationId": "1235134516",
                    "result": "success",
                    "resultReason": "",
                    "activityDisplayName": "Update policy",
                    "activityDateTime": "2024-11-27T03:31:26.2934305+00:00",
                    "loggedByService": "Core Directory",
                    "operationType": "Update",
                    "userAgent": None,
                    "initiatedBy": {
                        "user": {
                            "id": "1324512355",
                            "displayName": None,
                            "userPrincipalName": "Kratos@onmicrosoft.com",
                            "ipAddress": "1.2.3.4",
                            "roles": [],
                        },
                    },
                    "targetResources": [
                        {
                            "id": "12351254",
                            "displayName": "MFA",
                            "type": "Policy",
                            "modifiedProperties": [
                                {
                                    "displayName": "PolicyDetail",
                                    "oldValue": '["{\\"Version\\":1,\\"CreatedDateTime\\":\\"2024-11-21T16:48:48.1196443Z\\",\\"ModifiedDateTime\\":\\"2024-11-21T16:56:13.9120766Z\\",\\"State\\":\\"Enabled\\",\\"Conditions\\":{\\"Applications\\":{\\"Include\\":[{\\"Applications\\":[\\"None\\"]}]},\\"Users\\":{\\"Include\\":[{\\"Users\\":[\\"All\\"]}]}},\\"Controls\\":[{\\"Control\\":[\\"Mfa\\"]}],\\"SessionControls\\":[\\"SignInFrequency\\"],\\"SignInFrequencyTimeSpan\\":\\"90.00:00:00\\",\\"SignInFrequencyType\\":10,\\"EnforceAllPoliciesForEas\\":true,\\"IncludeOtherLegacyClientTypeForEvaluation\\":true}"]',
                                    "newValue": '["{\\"Version\\":1,\\"CreatedDateTime\\":\\"2024-11-21T16:48:48.1196443Z\\",\\"ModifiedDateTime\\":\\"2024-11-27T03:31:25.4989035Z\\",\\"State\\":\\"Enabled\\",\\"Conditions\\":{\\"Applications\\":{\\"Include\\":[{\\"Applications\\":[\\"None\\"]}]},\\"Users\\":{\\"Include\\":[{\\"Users\\":[\\"All\\"]}]}},\\"SessionControls\\":[\\"SignInFrequency\\"],\\"SignInFrequencyTimeSpan\\":\\"90.00:00:00\\",\\"SignInFrequencyType\\":10,\\"EnforceAllPoliciesForEas\\":true,\\"IncludeOtherLegacyClientTypeForEvaluation\\":true}"]',
                                },
                                {
                                    "displayName": "Included Updated Properties",
                                    "oldValue": None,
                                    "newValue": '"PolicyDetail"',
                                },
                            ],
                            "administrativeUnits": [],
                        },
                    ],
                    "additionalDetails": [{"key": "User-Agent", "value": "Microsoft Azure Graph Client Library 1.0"}],
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.slack import slack_alert_context


@panther_managed
class SlackAuditLogsAppAdded(Rule):
    id = "Slack.AuditLogs.AppAdded-prototype"
    display_name = "Slack App Added"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = ["Slack", "Persistence", "Server Software Component"]
    reports = {"MITRE ATT&CK": ["TA0003:T1505"]}
    default_severity = Severity.MEDIUM
    default_description = "Detects when a Slack App has been added to a workspace"
    default_reference = "https://slack.com/intl/en-gb/help/articles/202035138-Add-apps-to-your-Slack-workspace"
    summary_attributes = ["p_any_ip_addresses", "p_any_emails"]
    APP_ADDED_ACTIONS = ["app_approved", "app_installed", "org_app_workspace_added"]

    def rule(self, event):
        return event.get("action") in self.APP_ADDED_ACTIONS

    def title(self, event):
        return f"Slack App [{event.deep_get('entity', 'app', 'name')}] Added by [{event.deep_get('actor', 'user', 'name')}]"

    def alert_context(self, event):
        context = slack_alert_context(event)
        context["scopes"] = event.deep_get("entity", "scopes")
        return context

    def severity(self, event):
        # Used to escalate to High/Critical if the app is granted admin privileges
        # May want to escalate to "Critical" depending on security posture
        if "admin" in event.deep_get("entity", "app", "scopes", default=[]):
            return "High"
        # Fallback method in case the admin scope is not directly mentioned in entity for whatever
        if "admin" in event.deep_get("details", "new_scope", default=[]):
            return "High"
        if "admin" in event.deep_get("details", "bot_scopes", default=[]):
            return "High"
        return "Medium"

    tests = [
        RuleTest(
            name="App added to workspace - Admin not in app scopes",
            expected_result=True,
            log={
                "action": "app_installed",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "E012MH3HS94",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-panther-1",
                        "id": "T01770N79GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Go-http-client/2.0",
                },
                "date_create": "2021-06-08 22:16:15",
                "details": {"is_internal_integration": False, "is_token_rotation_enabled_app": False},
                "entity": {
                    "app": {
                        "id": "A049JV0H0KC",
                        "is_directory_approved": True,
                        "is_distributed": True,
                        "name": "Notion",
                        "scopes": [
                            "channels:history",
                            "channels:read",
                            "chat:write",
                            "groups:read",
                            "groups:write",
                            "im:read",
                            "mpim:read",
                            "groups:history",
                            "im:history",
                            "mpim:history",
                        ],
                    },
                    "type": "app",
                },
            },
        ),
        RuleTest(
            name="App Approved",
            expected_result=True,
            log={
                "action": "app_installed",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "E012MH3HS94",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-panther-1",
                        "id": "T01770N79GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Go-http-client/2.0",
                },
                "date_create": "2021-06-08 22:16:15",
                "details": {"app_owner_id": "W012J3AEWAU", "is_internal_integration": True},
                "entity": {
                    "app": {
                        "id": "A012F34BFEF",
                        "is_directory_approved": False,
                        "is_distributed": False,
                        "name": "app-name",
                        "scopes": ["admin"],
                    },
                    "type": "app",
                },
            },
        ),
        RuleTest(
            name="App Installed",
            expected_result=True,
            log={
                "action": "app_installed",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "E012MH3HS94",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-panther-1",
                        "id": "T01770N79GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Go-http-client/2.0",
                },
                "date_create": "2021-06-08 22:16:15",
                "details": {"app_owner_id": "W012J3AEWAU", "is_internal_integration": True},
                "entity": {
                    "app": {
                        "id": "A012F34BFEF",
                        "is_directory_approved": False,
                        "is_distributed": False,
                        "name": "app-name",
                        "scopes": ["admin"],
                    },
                    "type": "app",
                },
            },
        ),
        RuleTest(
            name="App added to workspace",
            expected_result=True,
            log={
                "action": "app_installed",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "E012MH3HS94",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-panther-1",
                        "id": "T01770N79GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Go-http-client/2.0",
                },
                "date_create": "2021-06-08 22:16:15",
                "details": {"app_owner_id": "W012J3AEWAU", "is_internal_integration": True},
                "entity": {
                    "app": {
                        "id": "A012F34BFEF",
                        "is_directory_approved": False,
                        "is_distributed": False,
                        "name": "app-name",
                        "scopes": ["admin"],
                    },
                    "type": "app",
                },
            },
        ),
        RuleTest(
            name="User Logout",
            expected_result=False,
            log={
                "action": "user_logout",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace-1",
                        "id": "T01234N56GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
                "date_create": "2022-07-28 15:22:32",
                "entity": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
            },
        ),
    ]



import argparse
import importlib
import logging
import sys
from typing import Callable, Tuple

from gql.transport.exceptions import TransportServerError, TransportQueryError

from pypanther import testing, cli_output
from pypanther.custom_logging import setup_logging
from pypanther.setup_subparsers import (
    setup_get_rule_parser,
    setup_list_log_types_parser,
    setup_list_rules_parser,
    setup_test_parser,
    setup_upload_parser,
)
from pypanther.backend import util
from pypanther.command import standard_args
from pypanther.config import dynaconf_argparse_merge, setup_dynaconf


def run():
    setup_logging()

    parser = setup_parser()
    args = parser.parse_args()
    if args.command is None:
        parser.print_help()
        return None

    if args.debug:
        logging.getLogger().setLevel(logging.DEBUG)
    else:
        logging.getLogger("gql.transport.aiohttp").setLevel(logging.WARNING)

    config_file_settings = setup_dynaconf()
    dynaconf_argparse_merge(vars(args), config_file_settings)

    try:
        return_code, out = args.func(args)
    except util.BackendNotFoundException as err:
        logging.error('Backend not found: "%s"', err)  # noqa: TRY400
        return 1
    except TransportQueryError as err:
        for e in err.errors:
            if e.get("message", "") == "access denied":
                print(cli_output.failed("Unauthorized. Please check your API key permissions"))
                return 1
        print(cli_output.failed(err))
        return 1
    except TransportServerError as err:
        if err.code == 401:
            print(cli_output.failed("Unauthorized. Please check that your API key is valid"))
        else:
            print(cli_output.failed(err))
        return 1
    except Exception as err:  # pylint: disable=broad-except
        # Catch arbitrary exceptions without printing help message
        logging.warning('Unhandled exception: "%s"', err, exc_info=err, stack_info=True)
        logging.debug("Full error traceback:", exc_info=err)
        return 1

    if return_code > 0 and out:
        logging.error(out)
    elif return_code == 0 and out:
        logging.info(out)

    sys.exit(return_code)


def setup_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Command line tool for using Panther's Detections-as-Code V2.",
        prog="pypanther",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument("--debug", action="store_true", dest="debug")
    subparsers = parser.add_subparsers(dest="command", help="Available commands")

    # Upload command
    upload_parser = subparsers.add_parser(
        "upload",
        help="Upload a file",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    standard_args.for_public_api(upload_parser, required=False)
    setup_upload_parser(upload_parser)

    # Test command
    test_parser = subparsers.add_parser(
        "test",
        help="Run tests on all your rules. Test failures will be printed out, along with a summary of "
        "the rules that were run and the test results. If a rule had no tests, the rule is marked "
        "as 'skipped' from testing. A rule will have no tests if no tests were set on the rule or "
        "the rule has a @panther_managed decorator. Tests can still be added to @panther_managed "
        "rules and those new tests will be tested. Run with the verbose flag to see information on "
        "which tests passed and which rules were skipped.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    test_parser.set_defaults(func=testing.run)
    setup_test_parser(test_parser)

    # Version command
    version_parser = subparsers.add_parser(
        "version",
        help="version",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    version_parser.set_defaults(func=version)

    # Get command
    get_parser = subparsers.add_parser(
        "get",
        help="Get the class associated with a specific id. By default includes any registered overrides.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    get_parser.set_defaults(func=help_printer(get_parser))
    get_subparsers = get_parser.add_subparsers()
    get_rule_parser = get_subparsers.add_parser(
        name="rule",
        help="Get the class associated with a specific rule by id.  By default includes any registered overrides.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    setup_get_rule_parser(get_rule_parser)

    # List command
    list_parser = subparsers.add_parser(
        name="list",
        help="List managed or register content",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    list_parser.set_defaults(func=help_printer(list_parser))
    list_subparsers = list_parser.add_subparsers()
    list_rules_parser = list_subparsers.add_parser(
        name="rules",
        help="List panther managed and registered rules. "
        "Lists registered rules by default. "
        "Use --managed flag to show managed Panther rules",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    setup_list_rules_parser(list_rules_parser)
    list_log_types_parser = list_subparsers.add_parser(
        name="log-types",
        help="List panther managed log-types. A case-insensitive substring can be provided to filter the results (e.g list log-types zee).",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    setup_list_log_types_parser(list_log_types_parser)

    return parser


def help_printer(parser: argparse.ArgumentParser) -> Callable[[argparse.Namespace], Tuple[int, str]]:
    """
    A helper function for printing help messages. To be used as a commands func when you want the help message
    to be printed when it is run. Useful for when things have subcommands and running the top command is meaningless.
    """

    def wrapper(_: argparse.Namespace) -> Tuple[int, str]:
        parser.print_help()
        return 0, ""

    return wrapper


def version(args):
    print(importlib.metadata.version("pypanther"))
    return 0, ""


if __name__ == "__main__":
    run()



from pypanther.rules.tines.tines_actions_disabled_changes import (
    TinesActionsDisabledChanges as TinesActionsDisabledChanges,
)
from pypanther.rules.tines.tines_custom_ca import TinesCustomCertificateAuthority as TinesCustomCertificateAuthority
from pypanther.rules.tines.tines_enqueued_retrying_job_deletion import (
    TinesEnqueuedRetryingJobDestruction as TinesEnqueuedRetryingJobDestruction,
)
from pypanther.rules.tines.tines_global_resource_destruction import (
    TinesGlobalResourceDestruction as TinesGlobalResourceDestruction,
)
from pypanther.rules.tines.tines_sso_settings import TinesSSOSettings as TinesSSOSettings
from pypanther.rules.tines.tines_story_items_destruction import TinesStoryItemsDestruction as TinesStoryItemsDestruction
from pypanther.rules.tines.tines_story_jobs_clearance import TinesStoryJobsClearance as TinesStoryJobsClearance
from pypanther.rules.tines.tines_team_destruction import TinesTeamDestruction as TinesTeamDestruction
from pypanther.rules.tines.tines_tenant_authtoken import TinesTenantAuthToken as TinesTenantAuthToken



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp import gcp_alert_context


@panther_managed
class GCPIAMserviceAccountsgetAccessTokenPrivilegeEscalation(Rule):
    id = "GCP.IAM.serviceAccounts.getAccessToken.Privilege.Escalation-prototype"
    display_name = "GCP IAM serviceAccounts getAccessToken Privilege Escalation"
    log_types = [LogType.GCP_AUDIT_LOG]
    reports = {"MITRE ATT&CK": ["TA0004:T1548"]}
    default_severity = Severity.HIGH
    default_description = "The Identity and Access Management (IAM) service manages authorization and authentication for a GCP environment. This means that there are very likely multiple privilege escalation methods that use the IAM service and/or its permissions."
    default_reference = "https://rhinosecuritylabs.com/gcp/privilege-escalation-google-cloud-platform-part-1/"

    def rule(self, event):
        authorization_info = event.deep_walk("protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if auth.get("permission") == "iam.serviceAccounts.getAccessToken" and auth.get("granted") is True:
                return True
        return False

    def title(self, event):
        actor = event.udm("actor_user")
        operation = event.deep_get("protoPayload", "methodName", default="<OPERATION_NOT_FOUND>")
        project_id = event.deep_get("resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] performed [{operation}] on project [{project_id}]"

    def alert_context(self, event):
        return gcp_alert_context(event)

    tests = [
        RuleTest(
            name="iam.serviceAccounts.getAccessToken granted",
            expected_result=True,
            log={
                "p_log_type": "GCP.AuditLog",
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "status": {},
                    "authenticationInfo": {
                        "principalEmail": "some-project@company.iam.gserviceaccount.com",
                        "serviceAccountKeyName": "//iam.googleapis.com/projects/some-project/serviceAccounts/some-project@company.iam.gserviceaccount.com/keys/a378358365ff3d22e9c1a72fecf4605ddff76b47",
                        "principalSubject": "serviceAccount:some-project@company.iam.gserviceaccount.com",
                    },
                    "requestMetadata": {
                        "callerIp": "1.2.3.4",
                        "requestAttributes": {"time": "2024-02-26T17:15:16.327542536Z", "auth": {}},
                        "destinationAttributes": {},
                    },
                    "serviceName": "iamcredentials.googleapis.com",
                    "methodName": "SignJwt",
                    "authorizationInfo": [
                        {"permission": "iam.serviceAccounts.getAccessToken", "granted": True, "resourceAttributes": {}},
                    ],
                    "resourceName": "projects/-/serviceAccounts/114885146936855121342",
                    "request": {
                        "name": "projects/-/serviceAccounts/some-project@company.iam.gserviceaccount.com",
                        "@type": "type.googleapis.com/google.iam.credentials.v1.SignJwtRequest",
                    },
                },
                "insertId": "1hu88qbef4d2o",
                "resource": {
                    "type": "service_account",
                    "labels": {
                        "project_id": "some-project",
                        "unique_id": "114885146936855121342",
                        "email_id": "some-project@company.iam.gserviceaccount.com",
                    },
                },
                "timestamp": "2024-02-26T17:15:16.314854637Z",
                "severity": "INFO",
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Fdata_access",
                "receiveTimestamp": "2024-02-26T17:15:17.100020459Z",
            },
        ),
        RuleTest(
            name="iam.serviceAccounts.getAccessToken not granted",
            expected_result=False,
            log={
                "p_log_type": "GCP.AuditLog",
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "status": {},
                    "authenticationInfo": {
                        "principalEmail": "some-project@company.iam.gserviceaccount.com",
                        "serviceAccountKeyName": "//iam.googleapis.com/projects/some-project/serviceAccounts/some-project@company.iam.gserviceaccount.com/keys/a378358365ff3d22e9c1a72fecf4605ddff76b47",
                        "principalSubject": "serviceAccount:some-project@company.iam.gserviceaccount.com",
                    },
                    "requestMetadata": {
                        "callerIp": "1.2.3.4",
                        "requestAttributes": {"time": "2024-02-26T17:15:16.327542536Z", "auth": {}},
                        "destinationAttributes": {},
                    },
                    "serviceName": "iamcredentials.googleapis.com",
                    "methodName": "SignJwt",
                    "authorizationInfo": [
                        {
                            "permission": "iam.serviceAccounts.getAccessToken",
                            "granted": False,
                            "resourceAttributes": {},
                        },
                    ],
                    "resourceName": "projects/-/serviceAccounts/114885146936855121342",
                    "request": {
                        "name": "projects/-/serviceAccounts/some-project@company.iam.gserviceaccount.com",
                        "@type": "type.googleapis.com/google.iam.credentials.v1.SignJwtRequest",
                    },
                },
                "insertId": "1hu88qbef4d2o",
                "resource": {
                    "type": "service_account",
                    "labels": {
                        "project_id": "some-project",
                        "unique_id": "114885146936855121342",
                        "email_id": "some-project@company.iam.gserviceaccount.com",
                    },
                },
                "timestamp": "2024-02-26T17:15:16.314854637Z",
                "severity": "INFO",
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Fdata_access",
                "receiveTimestamp": "2024-02-26T17:15:17.100020459Z",
            },
        ),
        RuleTest(
            name="Principal Subject Used",
            expected_result=True,
            log={
                "p_log_type": "GCP.AuditLog",
                "insertId": "1dy9ihte4iyjz",
                "logName": "projects/mylogs/logs/cloudaudit.googleapis.com%2Fdata_access",
                "operation": {
                    "first": True,
                    "id": "10172500524907939495",
                    "last": True,
                    "producer": "iamcredentials.googleapis.com",
                },
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {
                        "principalSubject": "example_principal_subject",
                        "serviceAccountDelegationInfo": [{}],
                    },
                    "authorizationInfo": [
                        {"granted": True, "permission": "iam.serviceAccounts.getAccessToken", "resourceAttributes": {}},
                    ],
                    "metadata": {
                        "identityDelegationChain": ["projects/-/serviceAccounts/xxxxx.iam.gserviceaccount.com"],
                    },
                    "methodName": "GenerateAccessToken",
                    "request": {
                        "@type": "type.googleapis.com/google.iam.credentials.v1.GenerateAccessTokenRequest",
                        "name": "projects/-/serviceAccounts/xxxxx.iam.gserviceaccount.com",
                    },
                    "requestMetadata": {
                        "callerIP": "gce-internal-ip",
                        "callerSuppliedUserAgent": "google-api-go-client/0.5 gke-metadata-server,gzip(gfe)",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2024-08-29T19:30:36.353462175Z"},
                    },
                    "resourceName": "projects/-/serviceAccounts/11111222223333444455",
                    "serviceName": "iamcredentials.googleapis.com",
                    "status": {},
                },
                "receiveTimestamp": "2024-08-29 19:30:36.424542070",
                "resource": {
                    "labels": {
                        "email_id": "sample@email.com",
                        "project_id": "sample_project_id",
                        "unique_id": "11111222223333444455",
                    },
                    "type": "service_account",
                },
                "severity": "INFO",
                "timestamp": "2024-08-29 19:30:36.339983306",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_guardduty_context


@panther_managed
class AWSGuardDutyHighSeverityFinding(Rule):
    id = "AWS.GuardDuty.HighSeverityFinding-prototype"
    display_name = "AWS GuardDuty High Severity Finding"
    log_types = [LogType.AWS_GUARDDUTY]
    tags = ["AWS"]
    default_severity = Severity.HIGH
    default_description = "A high-severity GuardDuty finding has been identified.\n"
    default_runbook = "Search related logs to understand the root cause of the activity. Search the Panther Summary Attribute type value in https://docs.aws.amazon.com/guardduty/latest/ug/guardduty_finding-types-active.html for additional details.\n"
    default_reference = (
        "https://docs.aws.amazon.com/guardduty/latest/ug/guardduty_findings.html#guardduty_findings-severity"
    )
    summary_attributes = ["severity", "type", "title", "p_any_domain_names", "p_any_aws_arns", "p_any_aws_account_ids"]

    def rule(self, event):
        if event.deep_get("service", "additionalInfo", "sample"):
            # in case of sample data
            # https://docs.aws.amazon.com/guardduty/latest/ug/sample_findings.html
            return False
        return 7.0 <= float(event.get("severity", 0)) <= 8.9

    def title(self, event):
        return event.get("title")

    def alert_context(self, event):
        return aws_guardduty_context(event)

    tests = [
        RuleTest(
            name="High Sev Finding",
            expected_result=True,
            log={
                "schemaVersion": "2.0",
                "accountId": "123456789012",
                "region": "us-east-1",
                "partition": "aws",
                "arn": "arn:aws:guardduty:us-west-2:123456789012:detector/111111bbbbbbbbbb5555555551111111/finding/90b82273685661b9318f078d0851fe9a",
                "type": "PrivilegeEscalation:IAMUser/AdministrativePermissions",
                "service": {
                    "serviceName": "guardduty",
                    "detectorId": "111111bbbbbbbbbb5555555551111111",
                    "action": {
                        "actionType": "AWS_API_CALL",
                        "awsApiCallAction": {
                            "api": "PutRolePolicy",
                            "serviceName": "iam.amazonaws.com",
                            "callerType": "Domain",
                            "domainDetails": {"domain": "cloudformation.amazonaws.com"},
                            "affectedResources": {"AWS::IAM::Role": "arn:aws:iam::123456789012:role/IAMRole"},
                        },
                    },
                    "resourceRole": "TARGET",
                    "additionalInfo": {},
                    "evidence": None,
                    "eventFirstSeen": "2020-02-14T17:59:17Z",
                    "eventLastSeen": "2020-02-14T17:59:17Z",
                    "archived": False,
                    "count": 1,
                },
                "severity": 8,
                "id": "eeb88ab56556eb7771b266670dddee5a",
                "createdAt": "2020-02-14T18:12:22.316Z",
                "updatedAt": "2020-02-14T18:12:22.316Z",
                "title": "Principal AssumedRole:IAMRole attempted to add a policy to themselves that is highly permissive.",
                "description": "Principal AssumedRole:IAMRole attempted to add a highly permissive policy to themselves.",
            },
        ),
        RuleTest(
            name="High Sev Finding As Sample Data",
            expected_result=False,
            log={
                "schemaVersion": "2.0",
                "accountId": "123456789012",
                "region": "us-east-1",
                "partition": "aws",
                "arn": "arn:aws:guardduty:us-west-2:123456789012:detector/111111bbbbbbbbbb5555555551111111/finding/90b82273685661b9318f078d0851fe9a",
                "type": "PrivilegeEscalation:IAMUser/AdministrativePermissions",
                "service": {
                    "serviceName": "guardduty",
                    "detectorId": "111111bbbbbbbbbb5555555551111111",
                    "action": {
                        "actionType": "AWS_API_CALL",
                        "awsApiCallAction": {
                            "api": "PutRolePolicy",
                            "serviceName": "iam.amazonaws.com",
                            "callerType": "Domain",
                            "domainDetails": {"domain": "cloudformation.amazonaws.com"},
                            "affectedResources": {"AWS::IAM::Role": "arn:aws:iam::123456789012:role/IAMRole"},
                        },
                    },
                    "resourceRole": "TARGET",
                    "additionalInfo": {"sample": True},
                    "evidence": None,
                    "eventFirstSeen": "2020-02-14T17:59:17Z",
                    "eventLastSeen": "2020-02-14T17:59:17Z",
                    "archived": False,
                    "count": 1,
                },
                "severity": 8,
                "id": "eeb88ab56556eb7771b266670dddee5a",
                "createdAt": "2020-02-14T18:12:22.316Z",
                "updatedAt": "2020-02-14T18:12:22.316Z",
                "title": "Principal AssumedRole:IAMRole attempted to add a policy to themselves that is highly permissive.",
                "description": "Principal AssumedRole:IAMRole attempted to add a highly permissive policy to themselves.",
            },
        ),
    ]



from pypanther.rules.cisco_umbrella_dns.domain_blocked import CiscoUmbrellaDNSBlocked as CiscoUmbrellaDNSBlocked
from pypanther.rules.cisco_umbrella_dns.fuzzy_matching_domains import (
    CiscoUmbrellaDNSFuzzyMatching as CiscoUmbrellaDNSFuzzyMatching,
)
from pypanther.rules.cisco_umbrella_dns.suspicious_domains import (
    CiscoUmbrellaDNSSuspicious as CiscoUmbrellaDNSSuspicious,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class PantherSAMLModified(Rule):
    id = "Panther.SAML.Modified-prototype"
    display_name = "Panther SAML configuration has been modified"
    log_types = [LogType.PANTHER_AUDIT]
    default_severity = Severity.HIGH
    tags = ["DataModel", "Defense Evasion:Impair Defenses"]
    reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    default_description = "An Admin has modified Panther's SAML configuration."
    default_runbook = "Ensure this change was approved and appropriate."
    default_reference = "https://docs.panther.com/system-configuration/saml"
    summary_attributes = ["p_any_ip_addresses", "p_any_usernames"]

    def rule(self, event):
        return event.get("actionName") == "UPDATE_SAML_SETTINGS" and event.get("actionResult") == "SUCCEEDED"

    def title(self, event):
        return f"Panther SAML config has been modified by {event.udm('actor_user')}"

    def alert_context(self, event):
        return {"user": event.udm("actor_user"), "ip": event.udm("source_ip")}

    tests = [
        RuleTest(
            name="SAML config modified",
            expected_result=True,
            log={
                "actionName": "UPDATE_SAML_SETTINGS",
                "actionParams": {},
                "actionResult": "SUCCEEDED",
                "actor": {
                    "attributes": {"email": "homer@springfield.gov", "emailVerified": True, "roleId": "111111"},
                    "id": "111111",
                    "name": "Homer Simpson",
                    "type": "USER",
                },
                "errors": None,
                "p_log_type": "Panther.Audit",
            },
        ),
        RuleTest(
            name="SAML config viewed",
            expected_result=False,
            log={
                "actionName": "GET_SAML_SETTINGS",
                "actionParams": {},
                "actionResult": "SUCCEEDED",
                "actor": {
                    "attributes": {"email": "homer@springfield.gov", "emailVerified": True, "roleId": "111111"},
                    "id": "111111",
                    "name": "Homer Simpson",
                    "type": "USER",
                },
                "errors": None,
                "p_log_type": "Panther.Audit",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.ipinfo import geoinfo_from_ip_formatted


@panther_managed
class AWSConsoleRootLogin(Rule):
    id = "AWS.Console.RootLogin-prototype"
    display_name = "Root Console Login"
    dedup_period_minutes = 15
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = [
        "AWS",
        "Identity & Access Management",
        "Authentication",
        "DemoThreatHunting",
        "Privilege Escalation:Valid Accounts",
    ]
    reports = {"CIS": ["3.6"], "MITRE ATT&CK": ["TA0004:T1078"]}
    default_severity = Severity.HIGH
    default_description = "The root account has been logged into."
    default_runbook = "Investigate the usage of the root account. If this root activity was not authorized, immediately change the root credentials and investigate what actions the root account took.\n"
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/id_root-user.html"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]

    def rule(self, event):
        return (
            event.get("eventName") == "ConsoleLogin"
            and event.deep_get("userIdentity", "type") == "Root"
            and (event.deep_get("responseElements", "ConsoleLogin") == "Success")
        )

    def title(self, event):
        return f"AWS root login detected from ({geoinfo_from_ip_formatted(event, 'sourceIPAddress')}) in account [{event.get('recipientAccountId')}]"

    def dedup(self, event):
        # Each Root login should generate a unique alert
        return "-".join([event.get("recipientAccountId"), event.get("eventName"), event.get("eventTime")])

    def alert_context(self, event):
        return {
            "sourceIPAddress": event.get("sourceIPAddress"),
            "userIdentityAccountId": event.deep_get("userIdentity", "accountId"),
            "userIdentityArn": event.deep_get("userIdentity", "arn"),
            "eventTime": event.get("eventTime"),
            "mfaUsed": event.deep_get("additionalEventData", "MFAUsed"),
        }

    tests = [
        RuleTest(
            name="Successful Root Login",
            expected_result=True,
            mocks=[
                RuleMock(
                    object_name="geoinfo_from_ip_formatted",
                    return_value="111.111.111.111 in SF, California in USA",
                ),
            ],
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "Root",
                    "principalId": "1111",
                    "arn": "arn:aws:iam::123456789012:root",
                    "accountId": "123456789012",
                    "userName": "root",
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "signin.amazonaws.com",
                "eventName": "ConsoleLogin",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla",
                "requestParameters": None,
                "responseElements": {"ConsoleLogin": "Success"},
                "additionalEventData": {
                    "LoginTo": "https://console.aws.amazon.com/console/",
                    "MobileVersion": "No",
                    "MFAUsed": "No",
                },
                "eventID": "1",
                "eventType": "AwsConsoleSignIn",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="Non-Login Event",
            expected_result=False,
            log={
                "eventVersion": "1.06",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:user/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:user/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "dynamodb.amazonaws.com",
                "eventName": "DescribeTable",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"tableName": "table"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "readOnly": True,
                "resources": [
                    {"accountId": "123456789012", "type": "AWS::DynamoDB::Table", "ARN": "arn::::table/table"},
                ],
                "eventType": "AwsApiCall",
                "apiVersion": "2012-08-10",
                "managementEvent": True,
                "recipientAccountId": "123456789012",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GSuiteCalendarMadePublic(Rule):
    id = "GSuite.CalendarMadePublic-prototype"
    display_name = "GSuite Calendar Has Been Made Public"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    reports = {"MITRE ATT&CK": ["TA0007:T1087"]}
    default_severity = Severity.MEDIUM
    default_description = "A User or Admin Has Modified A Calendar To Be Public\n"
    default_reference = "https://support.google.com/calendar/answer/37083?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Follow up with user about this calendar share.\n"
    summary_attributes = ["actor:email"]

    def rule(self, event):
        return (
            event.get("name") == "change_calendar_acls"
            and event.get("parameters", {}).get("grantee_email") == "__public_principal__@public.calendar.google.com"
        )

    def title(self, event):
        return f"GSuite calendar [{event.deep_get('parameters', 'calendar_id', default='<NO_CALENDAR_ID>')}] made {self.public_or_private(event)} by [{event.deep_get('actor', 'email', default='<NO_ACTOR_FOUND>')}]"

    def severity(self, event):
        return "LOW" if self.public_or_private(event) == "private" else "MEDIUM"

    def public_or_private(self, event):
        return "private" if event.deep_get("parameters", "access_level") == "none" else "public"

    tests = [
        RuleTest(
            name="User Publically Shared a Calendar",
            expected_result=True,
            log={
                "actor": {"email": "user@example.io", "profileId": "110111111111111111111"},
                "id": {
                    "applicationName": "calendar",
                    "customerId": "D12345",
                    "time": "2022-12-10 22:33:31.852000000",
                    "uniqueQualifier": "-2888888888888888888",
                },
                "ipAddress": "1.2.3.4",
                "kind": "admin#reports#activity",
                "name": "change_calendar_acls",
                "ownerDomain": "example.io",
                "parameters": {
                    "access_level": "freebusy",
                    "api_kind": "web",
                    "calendar_id": "user@example.io",
                    "grantee_email": "__public_principal__@public.calendar.google.com",
                    "user_agent": "Mozilla/5.0",
                },
                "type": "calendar_change",
            },
        ),
        RuleTest(
            name="User Made Calendar Private",
            expected_result=True,
            log={
                "actor": {"email": "user@example.io", "profileId": "110111111111111111111"},
                "id": {
                    "applicationName": "calendar",
                    "customerId": "D12345",
                    "time": "2022-12-10 22:33:31.852000000",
                    "uniqueQualifier": "-2888888888888888888",
                },
                "ipAddress": "1.2.3.4",
                "kind": "admin#reports#activity",
                "name": "change_calendar_acls",
                "ownerDomain": "example.io",
                "parameters": {
                    "access_level": "none",
                    "api_kind": "web",
                    "calendar_id": "user@example.io",
                    "grantee_email": "__public_principal__@public.calendar.google.com",
                    "user_agent": "Mozilla/5.0",
                },
                "type": "calendar_change",
            },
        ),
        RuleTest(
            name="Admin Set Default Calendar SHARING_OUTSIDE_DOMAIN Setting to READ_WRITE_ACCESS",
            expected_result=False,
            log={
                "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
                "id": {
                    "applicationName": "admin",
                    "customerId": "D12345",
                    "time": "2022-12-11 01:06:26.303000000",
                    "uniqueQualifier": "-12345",
                },
                "ipAddress": "12.12.12.12",
                "kind": "admin#reports#activity",
                "name": "CHANGE_CALENDAR_SETTING",
                "parameters": {
                    "DOMAIN_NAME": "example.io",
                    "NEW_VALUE": "READ_WRITE_ACCESS",
                    "OLD_VALUE": "READ_ONLY_ACCESS",
                    "ORG_UNIT_NAME": "Example IO",
                    "SETTING_NAME": "SHARING_OUTSIDE_DOMAIN",
                },
                "type": "CALENDAR_SETTINGS",
            },
        ),
        RuleTest(
            name="ListObject Type",
            expected_result=False,
            log={
                "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
                "id": {
                    "applicationName": "drive",
                    "customerId": "D12345",
                    "time": "2022-12-20 17:27:47.080000000",
                    "uniqueQualifier": "-7312729053723258069",
                },
                "ipAddress": "12.12.12.12",
                "kind": "admin#reports#activity",
                "name": "rename",
                "parameters": {
                    "actor_is_collaborator_account": None,
                    "billable": True,
                    "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                    "doc_title": "Document Title- Found Here",
                    "doc_type": "presentation",
                    "is_encrypted": None,
                    "new_value": ["Document Title- Found Here"],
                    "old_value": ["Document Title- Old"],
                    "owner": "user@example.io",
                    "owner_is_shared_drive": None,
                    "owner_is_team_drive": None,
                    "primary_event": True,
                    "visibility": "private",
                },
                "type": "access",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSEC2StartupScriptChange(Rule):
    default_description = "Detects changes to the EC2 instance startup script. The shell script will be executed as root/SYSTEM every time the specific instances are booted up."
    display_name = "AWS EC2 Startup Script Change"
    reports = {"MITRE ATT&CK": ["TA0002:T1059"]}
    default_reference = "https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/user-data.html#user-data-shell-scripts"
    default_severity = Severity.HIGH
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.EC2.Startup.Script.Change-prototype"

    def rule(self, event):
        if event.get("eventName") == "ModifyInstanceAttribute" and event.deep_get("requestParameters", "userData"):
            return True
        return False

    def title(self, event):
        return f"[{event.deep_get('userIdentity', 'arn')}] modified the startup script for  [{event.deep_get('requestParameters', 'instanceId')}] in [{event.get('recipientAccountId')}] - [{event.get('awsRegion')}]"

    def dedup(self, event):
        return event.deep_get("requestParameters", "instanceId")

    def alert_context(self, event):
        context = aws_rule_context(event)
        context["instance_ids"] = [event.deep_get("requestParameters", "instanceId"), "no_instance_id"]
        return context

    tests = [
        RuleTest(
            name="ModifyInstanceAttribute-NoUserData",
            expected_result=False,
            log={
                "awsregion": "us-east-1",
                "eventid": "abc-123",
                "eventname": "ModifyInstanceAttribute",
                "eventsource": "ec2.amazonaws.com",
                "eventtime": "2022-07-17 04:50:23",
                "eventtype": "AwsApiCall",
                "eventversion": "1.08",
                "p_any_aws_instance_ids": ["testinstanceid"],
                "p_event_time": "2022-07-17 04:50:23",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2022-07-17 04:55:11.788",
                "requestParameters": {"instanceId": "testinstanceid", "instanceType": {"value": "t3.nano"}},
            },
        ),
        RuleTest(
            name="ModifyInstanceAttributeUserdata",
            expected_result=True,
            log={
                "awsRegion": "us-east-2",
                "eventCategory": "Management",
                "eventName": "ModifyInstanceAttribute",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2022-09-30 15:11:25.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "0123456789",
                "requestID": "example-id",
                "requestParameters": {"instanceId": "i-012345abcde", "userData": "<sensitiveDataRemoved>"},
                "responseElements": {"_return": True, "requestId": "012345abcdef"},
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                    "clientProvidedHostHeader": "ec2.us-east-2.amazonaws.com",
                    "tlsVersion": "TLSv1.2",
                },
                "userAgent": "aws sdk",
                "userIdentity": {
                    "accessKeyId": "ABCDEXAMPLE123",
                    "accountId": "0123456789",
                    "arn": "arn:aws:sts::0123456789:assumed-role/Role-us-1/CodeBuild",
                    "principalId": "ABCDEXAMPLE:CodeBuild",
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-09-30T14:52:26Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "0123456789",
                            "arn": "arn:aws:iam::0123456789:role/CodeBuild-US-East",
                            "principalId": "AROAQUW22FGSKBTQ7R5HP",
                            "type": "Role",
                            "userName": "CodeBuild-US-East",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="NoModifyInstanceAttribute",
            expected_result=False,
            log={
                "awsregion": "us-east-1",
                "eventid": "abc-123",
                "eventname": "ModifyImageAttribute",
                "eventsource": "ec2.amazonaws.com",
                "eventtime": "2022-07-17 04:50:23",
                "eventtype": "AwsApiCall",
                "eventversion": "1.08",
                "p_any_aws_instance_ids": ["testinstanceid"],
                "p_event_time": "2022-07-17 04:50:23",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2022-07-17 04:55:11.788",
                "requestParameters": {"instanceId": "testinstanceid", "instanceType": {"value": "t3.nano"}},
            },
        ),
    ]



from pypanther.rules.gcp_k8s.gcp_k8s_cron_job_created_or_modified import (
    GCPGKEKubernetesCronJobCreatedOrModified as GCPGKEKubernetesCronJobCreatedOrModified,
)
from pypanther.rules.gcp_k8s.gcp_k8s_exec_into_pod import GCPK8sExecIntoPod as GCPK8sExecIntoPod
from pypanther.rules.gcp_k8s.gcp_k8s_ioc_activity import GCPK8sIOCActivity as GCPK8sIOCActivity
from pypanther.rules.gcp_k8s.gcp_k8s_new_daemonset_deployed import (
    GCPK8sNewDaemonsetDeployed as GCPK8sNewDaemonsetDeployed,
)
from pypanther.rules.gcp_k8s.gcp_k8s_pod_attached_to_node_host_network import (
    GCPK8sPodAttachedToNodeHostNetwork as GCPK8sPodAttachedToNodeHostNetwork,
)
from pypanther.rules.gcp_k8s.gcp_k8s_pod_create_or_modify_host_path_vol_mount import (
    GCPK8SPodCreateOrModifyHostPathVolumeMount as GCPK8SPodCreateOrModifyHostPathVolumeMount,
)
from pypanther.rules.gcp_k8s.gcp_k8s_pod_using_host_pid_namespace import (
    GCPK8sPodUsingHostPIDNamespace as GCPK8sPodUsingHostPIDNamespace,
)
from pypanther.rules.gcp_k8s.gcp_k8s_privileged_pod_created import (
    GCPK8SPrivilegedPodCreated as GCPK8SPrivilegedPodCreated,
)
from pypanther.rules.gcp_k8s.gcp_k8s_service_type_node_port_deployed import (
    GCPK8SServiceTypeNodePortDeployed as GCPK8SServiceTypeNodePortDeployed,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.zscaler import zia_alert_context, zia_success


@panther_managed
class ZIALogsDownloaded(Rule):
    id = "ZIA.Logs.Downloaded-prototype"
    default_description = "This rule detects when ZIA Audit Logs were downloaded."
    display_name = "ZIA Logs Downloaded"
    default_runbook = "Verify that this change was planned. If not, make sure no sensitive information was leaked."
    default_reference = "https://help.zscaler.com/zia/about-audit-logs"
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0007:T1654"]}
    log_types = [LogType.ZSCALER_ZIA_ADMIN_AUDIT_LOG]

    def rule(self, event):
        if not zia_success(event):
            return False
        action = event.deep_get("event", "action", default="ACTION_NOT_FOUND")
        category = event.deep_get("event", "category", default="CATEGORY_NOT_FOUND")
        if action == "DOWNLOAD" and category == "AUDIT_LOGS":
            return True
        return False

    def title(self, event):
        return f"[Zscaler.ZIA]: Audit logs were downloaded by admin with id [{event.deep_get('event', 'adminid', default='<ADMIN_ID_NOT_FOUND>')}]"

    def alert_context(self, event):
        return zia_alert_context(event)

    tests = [
        RuleTest(
            name="Logs downloaded",
            expected_result=True,
            log={
                "event": {
                    "action": "DOWNLOAD",
                    "adminid": "admin@test.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "AUDIT_LOGS",
                    "clientip": "1.2.3.4",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {},
                    "preaction": {},
                    "recordid": "363",
                    "resource": "None",
                    "result": "SUCCESS",
                    "subcategory": "AUDIT_LOGS",
                    "time": "2024-11-04 16:31:24.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
        RuleTest(
            name="Other event",
            expected_result=False,
            log={
                "event": {
                    "action": "SIGN_IN",
                    "adminid": "admin@test.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "LOGIN",
                    "clientip": "1.2.3.4",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {},
                    "preaction": {},
                    "recordid": "354",
                    "resource": "None",
                    "result": "SUCCESS",
                    "subcategory": "LOGIN",
                    "time": "2024-11-04 16:27:37.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.github import github_alert_context


@panther_managed
class GitHubOrgModeratorsAdd(Rule):
    id = "GitHub.Org.Moderators.Add-prototype"
    display_name = "GitHub User Added to Org Moderators"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Initial Access:Supply Chain Compromise"]
    default_severity = Severity.MEDIUM
    default_description = "Detects when a user is added to a GitHub org's list of moderators."
    default_reference = "https://docs.github.com/en/organizations/managing-peoples-access-to-your-organization-with-roles/managing-moderators-in-your-organization"

    def rule(self, event):
        return event.get("action") == "organization_moderators.add_user"

    def title(self, event):
        return f"GitHub.Audit: User [{event.get('actor', '<UNKNOWN_ACTOR>')}] added user [{event.get('user', '<UNKNOWN_USER>')}] to moderators in [{event.get('org', '<UNKNOWN_ORG>')}]"

    def alert_context(self, event):
        return github_alert_context(event)

    tests = [
        RuleTest(
            name="GitHub - Org Moderator Added",
            expected_result=True,
            log={
                "_document_id": "Ab123",
                "action": "organization_moderators.add_user",
                "actor": "sarah78",
                "actor_location": {"country_code": "US"},
                "at_sign_timestamp": "2022-12-11 05:17:28.078",
                "created_at": "2022-12-11 05:17:28.078",
                "org": "example-io",
                "user": "john1987",
            },
        ),
        RuleTest(
            name="GitHub - Org Moderator removed",
            expected_result=False,
            log={
                "_document_id": "Ab123",
                "action": "organization_moderators.remove_user",
                "actor": "sarah78",
                "actor_location": {"country_code": "US"},
                "at_sign_timestamp": "2022-12-11 05:17:28.078",
                "created_at": "2022-12-11 05:17:28.078",
                "org": "example-io",
                "user": "john1987",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSCloudTrailDNSLogsDeleted(Rule):
    id = "AWS.CloudTrail.DNSLogsDeleted-prototype"
    display_name = "AWS DNS Logs Deleted"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    reports = {"MITRE ATT&CK": ["TA0005:T1562.008"]}
    default_description = "Detects when logs for a DNS Resolver have been removed."
    default_reference = "https://stratus-red-team.cloud/attack-techniques/AWS/aws.defense-evasion.dns-delete-logs/"
    default_runbook = "Determine if the log removal to is legitimate."
    tags = [
        "AWS",
        "Cloudtrail",
        "Defense Evasion",
        "Impair Defenses",
        "Disable or Modify Cloud Logs",
        "Defense Evasion:Impair Defenses",
        "Security Control",
        "Beta",
    ]

    def rule(self, event):
        return aws_cloudtrail_success(event) and event.get("eventName") == "DeleteResolverQueryLogConfig"

    def title(self, event):
        account = event.deep_get("userIdentity", "accountId", default="<UNKNOWN ACCOUNT>")
        region = event.get("awsRegion", "<UNKNOWN REGION>")
        return f"DNS logs have been deleted in {account} in {region}"

    def alert_context(self, event):
        log_id = event.deep_get("requestParameters", "resolverQueryLogConfigId", "<UNKNOWN LOG ID>")
        return aws_rule_context(event) | {"logId": log_id}

    tests = [
        RuleTest(
            name="Logs Deleted",
            expected_result=True,
            log={
                "p_event_time": "2024-11-27 18:18:58.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-11-27 18:25:54.213480847",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "27e6be30-7c86-4544-b0e0-a60b0c927887",
                "eventName": "DeleteResolverQueryLogConfig",
                "eventSource": "route53resolver.amazonaws.com",
                "eventTime": "2024-11-27 18:18:58.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "a45a0f04-8911-4c95-a9d7-3fead8a9bc45",
                "requestParameters": {"originSequenceNumber": 0, "resolverQueryLogConfigId": "rqlc-5aa596fe3bd84ec6"},
                "responseElements": {
                    "resolverQueryLogConfig": {
                        "arn": "arn:aws:route53resolver:us-west-2:111122223333:resolver-query-log-config/rqlc-5aa596fe3bd84ec6",
                        "associationCount": 0,
                        "creationTime": "2024-11-27T18:18:56.881520365Z",
                        "creatorRequestId": "tf-r53-resolver-query-log-config-20241127181856499800000001",
                        "destinationArn": "arn:aws:s3:::sample-bucket-name",
                        "id": "rqlc-5aa596fe3bd84ec6",
                        "name": "sample-config-name",
                        "ownerId": "111122223333",
                        "shareStatus": "NOT_SHARED",
                        "status": "DELETING",
                    },
                },
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "route53resolver.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "stratus-red-team_dbac929e-ae11-4539-8753-35dbcbbc3256",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/leroy.jenkins",
                    "principalId": "SAMPLE_PRINCIPAL_ID:leroy.jenkins",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-11-27T18:17:21Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.slack import slack_alert_context


@panther_managed
class SlackAuditLogsAppRemoved(Rule):
    id = "Slack.AuditLogs.AppRemoved-prototype"
    display_name = "Slack App Removed"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = ["Slack", "Impact", "Service Stop", "Defense Evasion", "Indicator Removal", "Clear Persistence"]
    reports = {"MITRE ATT&CK": ["TA0040:T1489", "TA0005:T1070.009"]}
    default_severity = Severity.MEDIUM
    default_description = "Detects when a Slack App has been removed"
    default_reference = "https://slack.com/intl/en-gb/help/articles/360003125231-Remove-apps-and-customised-integrations-from-your-workspace"
    summary_attributes = ["p_any_ip_addresses", "p_any_emails"]
    APP_REMOVED_ACTIONS = ["app_restricted", "app_uninstalled", "org_app_workspace_removed"]

    def rule(self, event):
        return event.get("action") in self.APP_REMOVED_ACTIONS

    def title(self, event):
        return f"Slack App [{event.deep_get('entity', 'app', 'name')}] Removed by [{event.deep_get('actor', 'user', 'name')}]"

    def alert_context(self, event):
        return slack_alert_context(event)

    tests = [
        RuleTest(
            name="App Restricted",
            expected_result=True,
            log={
                "action": "app_restricted",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "E012MH3HS94",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-panther-1",
                        "id": "T01770N79GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Go-http-client/2.0",
                },
                "date_create": "2021-06-08 22:16:15",
                "details": {"app_owner_id": "W012J3AEWAU", "is_internal_integration": True},
                "entity": {
                    "app": {
                        "id": "A012F34BFEF",
                        "is_directory_approved": False,
                        "is_distributed": False,
                        "name": "app-name",
                        "scopes": ["admin"],
                    },
                    "type": "app",
                },
            },
        ),
        RuleTest(
            name="App Uninstalled",
            expected_result=True,
            log={
                "action": "app_uninstalled",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "E012MH3HS94",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-panther-1",
                        "id": "T01770N79GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Go-http-client/2.0",
                },
                "date_create": "2021-06-08 22:16:15",
                "details": {"app_owner_id": "W012J3AEWAU", "is_internal_integration": True},
                "entity": {
                    "app": {
                        "id": "A012F34BFEF",
                        "is_directory_approved": False,
                        "is_distributed": False,
                        "name": "app-name",
                        "scopes": ["admin"],
                    },
                    "type": "app",
                },
            },
        ),
        RuleTest(
            name="App removed from workspace",
            expected_result=True,
            log={
                "action": "org_app_workspace_removed",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "E012MH3HS94",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-panther-1",
                        "id": "T01770N79GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Go-http-client/2.0",
                },
                "date_create": "2021-06-08 22:16:15",
                "details": {"app_owner_id": "W012J3AEWAU", "is_internal_integration": True},
                "entity": {
                    "app": {
                        "id": "A012F34BFEF",
                        "is_directory_approved": False,
                        "is_distributed": False,
                        "name": "app-name",
                        "scopes": ["admin"],
                    },
                    "type": "app",
                },
            },
        ),
        RuleTest(
            name="User Logout",
            expected_result=False,
            log={
                "action": "user_logout",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace-1",
                        "id": "T01234N56GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
                "date_create": "2022-07-28 15:22:32",
                "entity": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GSuiteAdvancedProtection(Rule):
    id = "GSuite.AdvancedProtection-prototype"
    display_name = "GSuite User Advanced Protection Change"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite", "Defense Evasion:Impair Defenses"]
    reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    default_severity = Severity.LOW
    default_description = "A user disabled advanced protection for themselves.\n"
    default_reference = "https://support.google.com/a/answer/9378686?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Have the user re-enable Google Advanced Protection\n"
    summary_attributes = ["actor:email"]

    def rule(self, event):
        if event.deep_get("id", "applicationName") != "user_accounts":
            return False
        return bool(event.get("name") == "titanium_unenroll")

    def title(self, event):
        return (
            f"Advanced protection was disabled for user [{event.deep_get('actor', 'email', default='<UNKNOWN_EMAIL>')}]"
        )

    tests = [
        RuleTest(
            name="Advanced Protection Enabled",
            expected_result=False,
            log={
                "id": {"applicationName": "user_accounts"},
                "actor": {"callerType": "USER", "email": "homer.simpson@example.com"},
                "type": "titanium_change",
                "name": "titanium_enroll",
            },
        ),
        RuleTest(
            name="Advanced Protection Disabled",
            expected_result=True,
            log={
                "id": {"applicationName": "user_accounts"},
                "actor": {"callerType": "USER", "email": "homer.simpson@example.com"},
                "type": "titanium_change",
                "name": "titanium_unenroll",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class OneLoginLogin(Rule):
    id = "OneLogin.Login-prototype"
    display_name = "Signal - OneLogin Login"
    create_alert = False
    log_types = [LogType.ONELOGIN_EVENTS]
    tags = ["OneLogin"]
    default_severity = Severity.INFO
    default_description = "A OneLogin user successfully logged in."
    default_reference = "https://resources.onelogin.com/OneLogin_RiskBasedAuthentication-WP-v5.pdf"

    def rule(self, event):
        if str(event.get("event_type_id")) == "5":
            return True
        return False

    def title(self, event):
        return f"A user [{event.get('user_name', '<UNKNOWN_USER>')}] successfully logged in"

    tests = [
        RuleTest(
            name="Successful Login Event",
            expected_result=True,
            log={
                "event_type_id": "5",
                "actor_user_id": 123456,
                "actor_user_name": "Bob Cat",
                "user_id": 123456,
                "user_name": "Bob Cat",
            },
        ),
        RuleTest(
            name="Failed Login Event",
            expected_result=False,
            log={
                "event_type_id": "6",
                "actor_user_id": 123456,
                "actor_user_name": "Bob Cat",
                "user_id": 123456,
                "user_name": "Bob Cat",
            },
        ),
    ]



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class SnowflakeStreamTableCopiedIntoStage(Rule):
    id = "Snowflake.Stream.TableCopiedIntoStage-prototype"
    display_name = "Snowflake Table Copied Into Stage"
    log_types = [LogType.SNOWFLAKE_QUERY_HISTORY]
    default_severity = Severity.INFO
    create_alert = False
    reports = {"MITRE ATT&CK": ["TA0010:T1041"]}
    default_description = "A table was copied into a stage."
    default_reference = (
        "https://cloud.google.com/blog/topics/threat-intelligence/unc5537-snowflake-data-theft-extortion/"
    )
    tags = ["Snowflake", "[MITRE] Exfiltration", "[MITRE] Exfiltration Over C2 Channel"]
    STAGE_EXPR = re.compile("COPY\\s+INTO\\s+(?:\\$\\$|')?@([\\w\\.]+)", flags=re.I)
    PATH_EXPR = re.compile("COPY\\s+INTO\\s+(?:\\$\\$|')?@([\\w\\./]+)(?:\\$\\$|')?\\s+FROM", flags=re.I)
    STAGE = ""

    def rule(self, event):
        self.STAGE = self.STAGE_EXPR.match(event.get("QUERY_TEXT", ""))
        return all(
            (event.get("QUERY_TYPE") == "UNLOAD", self.STAGE is not None, event.get("EXECUTION_STATUS") == "SUCCESS"),
        )

    def alert_context(self, event):
        path = self.PATH_EXPR.match(event.get("QUERY_TEXT", ""))
        return {"actor": event.get("USER_NAME"), "path": path.group(1), "stage": self.STAGE.group(1).lower()}

    tests = [
        RuleTest(
            name="Copy from Table into Stage",
            expected_result=True,
            log={
                "EXECUTION_STATUS": "SUCCESS",
                "QUERY_TEXT": "COPY INTO '@PANTHER_LOGS.PUBLIC.data_exfil/DATA.csv' FROM (SELECT * FROM PANTHER_LOGS.PUBLIC.amazon_eks_audit LIMIT 100) FILE_FORMAT = ( TYPE='CSV' COMPRESSION=GZIP FIELD_DELIMITER=',' ESCAPE=NONE ESCAPE_UNENCLOSED_FIELD=NONE date_format='AUTO' time_format='AUTO' timestamp_format='AUTO' binary_format='UTF-8' null_if='' EMPTY_FIELD_AS_NULL = FALSE ) overwrite=TRUE single=FALSE max_file_size=5368709120 header=TRUE",
                "QUERY_TYPE": "UNLOAD",
                "USER_NAME": "LEX_LUTHOR",
            },
        ),
        RuleTest(
            name="Copy from Stage into Table",
            expected_result=False,
            log={
                "EXECUTION_STATUS": "SUCCESS",
                "QUERY_TEXT": "COPY INTO mytable\nFROM @mystage/result/data_ FILE_FORMAT = (FORMAT_NAME='CSV' COMPRESSION='GZIP');",
                "QUERY_TYPE": "UNLOAD",
                "USER_NAME": "LEX_LUTHOR",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GSuiteGroupBannedUser(Rule):
    id = "GSuite.GroupBannedUser-prototype"
    display_name = "GSuite User Banned from Group"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.LOW
    default_description = "A GSuite user was banned from an enterprise group by moderator action.\n"
    default_reference = "https://support.google.com/a/users/answer/9303224?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Investigate the banned user to see if further disciplinary action needs to be taken.\n"
    summary_attributes = ["actor:email"]

    def rule(self, event):
        if event.deep_get("id", "applicationName") != "groups_enterprise":
            return False
        if event.get("type") == "moderator_action":
            return bool(event.get("name") == "ban_user_with_moderation")
        return False

    def title(self, event):
        return f"User [{event.deep_get('actor', 'email', default='<UNKNOWN_EMAIL>')}] banned another user from a group."

    tests = [
        RuleTest(
            name="User Added",
            expected_result=False,
            log={
                "id": {"applicationName": "groups_enterprise"},
                "actor": {"email": "homer.simpson@example.com"},
                "type": "moderator_action",
                "name": "add_member",
            },
        ),
        RuleTest(
            name="User Banned from Group",
            expected_result=True,
            log={
                "id": {"applicationName": "groups_enterprise"},
                "actor": {"email": "homer.simpson@example.com"},
                "type": "moderator_action",
                "name": "ban_user_with_moderation",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSCloudTrailVPCEAccessDenied(Rule):
    id = "AWS.CloudTrail.VPCE.AccessDenied-prototype"
    display_name = "VPC Endpoint Access Denied"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.MEDIUM
    tags = [
        "AWS",
        "VPC",
        "CloudTrail",
        "Network Boundary Bridging",
        "Defense Evasion",
        "Lateral Movement",
        "Impair Defenses",
    ]
    reports = {"MITRE ATT&CK": ["TA0005:T1599", "TA0007:T1526"]}
    default_description = "Detects when access is denied due to VPC Endpoint policies, which could indicate attempted unauthorized access to AWS resources."
    default_runbook = "1. Identify the principal (user/role) and source IP that was denied access 2. Determine if this is expected behavior based on your VPC endpoint policies 3. Check if there are multiple failed attempts from the same principal/IP 4. If unexpected, investigate why the principal is attempting to access resources through the VPC endpoint 5. Consider updating your VPC endpoint policies if necessary 6. Document findings and take appropriate remediation steps based on investigation\n"
    default_reference = "https://www.wiz.io/blog/aws-vpc-endpoint-cloudtrail"
    summary_attributes = [
        "errorCode",
        "errorMessage",
        "sourceIPAddress",
        "eventSource",
        "eventName",
        "userIdentity.principalId",
    ]

    def rule(self, event):
        # Check if this is a VPC Endpoint network activity event
        if event.get("eventType") != "AwsVpceEvent" or event.get("eventCategory") != "NetworkActivity":
            return False
        # Look for access denied errors
        if event.get("errorCode") == "VpceAccessDenied":
            return True
        return False

    def title(self, event):
        actor_user = event.udm("actor_user")
        source_ip = event.get("sourceIPAddress", "unknown")
        service = event.get("eventSource", "unknown").split(".")[0]
        return f"VPC Endpoint Access Denied for [{actor_user}] from [{source_ip}] to [{service}]"

    def alert_context(self, event):
        account_id = event.deep_get("userIdentity", "accountId", default="unknown")
        context = aws_rule_context(event)
        context.update(
            {
                "account_id": account_id,
                "principal_id": event.deep_get("userIdentity", "principalId", default="unknown"),
                "source_ip": event.get("sourceIPAddress", "unknown"),
                "event_source": event.get("eventSource", "unknown"),
                "api_call": event.get("eventName", "unknown"),
                "error_message": event.get("errorMessage", ""),
                "resources": event.get("resources", []),
                "actor_user": event.udm("actor_user"),
            },
        )
        return context

    tests = [
        RuleTest(
            name="VPC Endpoint Access Denied",
            expected_result=True,
            log={
                "eventVersion": "1.08",
                "eventCategory": "NetworkActivity",
                "eventType": "AwsVpceEvent",
                "errorCode": "VpceAccessDenied",
                "errorMessage": "The request was denied due to a VPC endpoint policy",
                "eventTime": "2023-03-01T00:00:00Z",
                "awsRegion": "us-east-1",
                "eventSource": "s3.amazonaws.com",
                "eventName": "GetObject",
                "sourceIPAddress": "10.0.0.1",
                "userIdentity": {
                    "type": "AWSAccount",
                    "principalId": "AROAEXAMPLE:session-name",
                    "accountId": "111111111111",
                },
                "recipientAccountId": "222222222222",
                "requestParameters": {"bucketName": "example-bucket", "key": "sensitive-file.txt"},
                "responseElements": None,
                "vpcEndpointId": "vpce-EXAMPLE08c1b6b9b7",
                "vpcEndpointAccountId": "222222222222",
            },
        ),
        RuleTest(
            name="Not VPC Endpoint Event",
            expected_result=False,
            log={
                "eventVersion": "1.08",
                "eventCategory": "Management",
                "eventType": "AwsApiCall",
                "errorCode": "AccessDenied",
                "errorMessage": "Access Denied",
                "eventTime": "2023-03-01T00:00:00Z",
                "awsRegion": "us-east-1",
                "eventSource": "s3.amazonaws.com",
                "eventName": "GetObject",
                "sourceIPAddress": "10.0.0.1",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "AROAEXAMPLE:session-name",
                    "accountId": "111111111111",
                },
                "recipientAccountId": "222222222222",
                "requestParameters": {"bucketName": "example-bucket", "key": "sensitive-file.txt"},
                "responseElements": None,
            },
        ),
        RuleTest(
            name="VPC Endpoint Event Without Error",
            expected_result=False,
            log={
                "eventVersion": "1.08",
                "eventCategory": "NetworkActivity",
                "eventType": "AwsVpceEvent",
                "eventTime": "2023-03-01T00:00:00Z",
                "awsRegion": "us-east-1",
                "eventSource": "s3.amazonaws.com",
                "eventName": "GetObject",
                "sourceIPAddress": "10.0.0.1",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "AROAEXAMPLE:session-name",
                    "accountId": "111111111111",
                },
                "recipientAccountId": "222222222222",
                "requestParameters": {"bucketName": "example-bucket", "key": "sensitive-file.txt"},
                "responseElements": None,
            },
        ),
    ]



from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.base import key_value_list_to_dict


@panther_managed
class GCPUserAddedToPrivilegedGroup(Rule):
    id = "GCP.User.Added.To.Privileged.Group-prototype"
    display_name = "GCP User Added to Privileged Group"
    enabled = False
    log_types = [LogType.GCP_AUDIT_LOG]
    default_severity = Severity.LOW
    tags = ["Configuration Required"]
    reports = {"MITRE ATT&CK": ["TA0004:T1078.004", "TA0004:T1484.001"]}
    default_description = "A user was added to a group with special previleges"
    default_reference = "https://github.com/GoogleCloudPlatform/security-analytics/blob/main/src/2.02/2.02.md"
    default_runbook = "Determine if the user had been added to the group for legitimate reasons."
    # "admins@example.com"
    PRIVILEGED_GROUPS = {}
    USER_EMAIL = ""
    GROUP_EMAIL = ""

    def rule(self, event):
        events = event.deep_get("protoPayload", "metadata", "event", default=[])
        for event_ in events:
            if event_.get("eventname") != "ADD_GROUP_MEMBER":
                continue
            # Get the username
            params = key_value_list_to_dict(event_.get("parameter", []), "name", "value")
            self.USER_EMAIL = params.get("USER_EMAIL")
            self.GROUP_EMAIL = params.get("GROUP_EMAIL")
            if self.GROUP_EMAIL in self.get_privileged_groups():
                return True
        return False

    def title(self, event):
        actor = event.deep_get("actor", "email", default="")
        return f"{actor} has added {self.USER_EMAIL} to the privileged group {self.GROUP_EMAIL}"

    def get_privileged_groups(self):
        # We make this a function, so we can mock it for unit tests
        return self.PRIVILEGED_GROUPS

    tests = [
        RuleTest(
            name="User Added to Privileged Group",
            expected_result=True,
            mocks=[RuleMock(object_name="get_privileged_groups", return_value='["admins@example.com"]')],
            log={
                "logName": "organizations/123/logs/cloudaudit.googleapis.com%2Factivity",
                "severity": "NOTICE",
                "insertId": "285djodxlmu",
                "resource": {
                    "type": "audited_resource",
                    "labels": {"method": "google.admin.AdminService.addGroupMember", "service": "admin.googleapis.com"},
                },
                "timestamp": "2022-03-22T22:12:58.916Z",
                "receiveTimestamp": "2022-03-22T22:12:59.439766009Z",
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "serviceName": "admin.googleapis.com",
                    "methodName": "google.admin.AdminService.addGroupMember",
                    "resourceName": "organizations/123/groupSettings",
                    "authenticationInfo": {"principalEmail": "admin@example.com"},
                    "requestMetadata": {
                        "callerIP": "11.22.33.44",
                        "requestAttributes": {},
                        "destinationAttributes": {},
                    },
                    "metadata": {
                        "@type": "type.googleapis.com/ccc_hosted_reporting.ActivityProto",
                        "activityId": {"timeUsec": "1647987178916000", "uniqQualifier": "-8614641986436885296"},
                        "event": [
                            {
                                "eventName": "ADD_GROUP_MEMBER",
                                "eventType": "GROUP_SETTINGS",
                                "parameter": [
                                    {
                                        "label": "LABEL_OPTIONAL",
                                        "value": "test-user@example.com",
                                        "type": "TYPE_STRING",
                                        "name": "USER_EMAIL",
                                    },
                                    {
                                        "type": "TYPE_STRING",
                                        "value": "admins@example.com",
                                        "label": "LABEL_OPTIONAL",
                                        "name": "GROUP_EMAIL",
                                    },
                                ],
                            },
                        ],
                    },
                },
            },
        ),
        RuleTest(
            name="User Added to Non-Privileged Group",
            expected_result=False,
            log={
                "logName": "organizations/123/logs/cloudaudit.googleapis.com%2Factivity",
                "severity": "NOTICE",
                "insertId": "285djodxlmu",
                "resource": {
                    "type": "audited_resource",
                    "labels": {"method": "google.admin.AdminService.addGroupMember", "service": "admin.googleapis.com"},
                },
                "timestamp": "2022-03-22T22:12:58.916Z",
                "receiveTimestamp": "2022-03-22T22:12:59.439766009Z",
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "serviceName": "admin.googleapis.com",
                    "methodName": "google.admin.AdminService.addGroupMember",
                    "resourceName": "organizations/123/groupSettings",
                    "authenticationInfo": {"principalEmail": "admin@example.com"},
                    "requestMetadata": {
                        "callerIP": "11.22.33.44",
                        "requestAttributes": {},
                        "destinationAttributes": {},
                    },
                    "metadata": {
                        "@type": "type.googleapis.com/ccc_hosted_reporting.ActivityProto",
                        "activityId": {"timeUsec": "1647987178916000", "uniqQualifier": "-8614641986436885296"},
                        "event": [
                            {
                                "eventName": "ADD_GROUP_MEMBER",
                                "eventType": "GROUP_SETTINGS",
                                "parameter": [
                                    {
                                        "label": "LABEL_OPTIONAL",
                                        "value": "test-user@example.com",
                                        "type": "TYPE_STRING",
                                        "name": "USER_EMAIL",
                                    },
                                    {
                                        "type": "TYPE_STRING",
                                        "value": "normies@example.com",
                                        "label": "LABEL_OPTIONAL",
                                        "name": "GROUP_EMAIL",
                                    },
                                ],
                            },
                        ],
                    },
                },
            },
        ),
    ]



"""
Used to define global helpers and variables for GSuite events.
"""
# # # # # # # # # # # # # #
#      GSuite Helpers     #
# # # # # # # # # # # # # #

GSUITE_PARAMETER_VALUES = [
    "value",
    "intValue",
    "boolValue",
    "multiValue",
    "multiIntValue",
    "messageValue",
    "multiMessageValue",
]


# GSuite parameters are formatted as a list of dictionaries, where each dictionary has a 'name' key
# that maps to the name of the parameter, and one key from GSUITE_PARAMETER_VALUES that maps to the
# value of the parameter. This means to lookup the value of a particular parameter, you must
# traverse the entire list of parameters to find it and then know (or guess) what type of value it
# contains. This helper function handles that for us.
#
# Example parameters list:
# parameters = [
#   {
#       "name": "event_id",
#       "value": "abc123"
#   },
#   {
#       "name": "start_time",
#       "intValue": 63731901000
#   },
#   {
#       "name": "end_time",
#       "intValue": 63731903000
#   },
#   {
#       "name": "things",
#       "multiValue": [ "DRIVE" , "MEME"]
#   }
# ]
def gsuite_parameter_lookup(parameters, key):
    for param in parameters:
        if param["name"] != key:
            continue
        for value in GSUITE_PARAMETER_VALUES:
            if value in param:
                return param[value]
        return None
    return None


# GSuite event details are formatted as a list of dictionaries. Each entry has a 'type' and 'name'.
#
# In order to find the event details of interest, you must loop through
# the list searching for a particular type and name.
#
# This helper function handles the looping functionality that is common in many of the gsuite rules
def gsuite_details_lookup(detail_type, detail_names, event):
    for details in event.get("events", {}):
        if details.get("type") == detail_type and details.get("name") in detail_names:
            return details
    # not found, return empty dict
    return {}



from pypanther.rules.aws_eks.anonymous_api_access import AmazonEKSAnonymousAPIAccess as AmazonEKSAnonymousAPIAccess
from pypanther.rules.aws_eks.source_ip_multiple_403 import AmazonEKSAuditMultiple403 as AmazonEKSAuditMultiple403
from pypanther.rules.aws_eks.system_namespace_public_ip import (
    AmazonEKSAuditSystemNamespaceFromPublicIP as AmazonEKSAuditSystemNamespaceFromPublicIP,
)



from pypanther.rules.intune.intune_create_modify_client_app import (
    IntuneCreateModifyClientApp as IntuneCreateModifyClientApp,
)
from pypanther.rules.intune.intune_device_not_compliant import IntuneDeviceNotCompliant as IntuneDeviceNotCompliant
from pypanther.rules.intune.intune_new_device_management_script import (
    IntuneNewDeviceManagementScript as IntuneNewDeviceManagementScript,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.okta import okta_alert_context


@panther_managed
class OktaGroupAdminRoleAssigned(Rule):
    default_description = "Detect when an admin role is assigned to a group"
    display_name = "Okta Group Admin Role Assigned"
    default_reference = "https://support.okta.com/help/s/article/How-to-assign-Administrator-roles-to-groups?language=en_US#:~:text=Log%20in%20to%20the%20Admin,user%20and%20click%20Save%20changes"
    default_severity = Severity.HIGH
    log_types = [LogType.OKTA_SYSTEM_LOG]
    id = "Okta.Group.Admin.Role.Assigned-prototype"

    def rule(self, event):
        return event.get("eventtype", "") == "group.privilege.grant"

    def title(self, event):
        # pylint: disable=W0613
        return f"Okta Admin Privileges Assigned to Group [{event.get('target', [{}])[0].get('alternateId', '<id-not-found>')}]"

    def alert_context(self, event):
        return okta_alert_context(event)

    tests = [
        RuleTest(
            name="Group Privilege Grant",
            expected_result=True,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpsons",
                    "id": "00ABC123",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "xyz1234"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Springfield",
                        "country": "United States",
                        "geolocation": {"lat": 11.111, "lon": -70},
                        "postalCode": "1234",
                        "state": "California",
                    },
                    "ipAddress": "1.2.3.4",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "authnRequestId": "ABC123",
                        "deviceFingerprint": "009988771ABC",
                        "dtHash": "123abc1234",
                        "requestId": "abc-111-adf",
                        "requestUri": "/idp/idx/identify",
                        "threatSuspected": "false",
                        "url": "/idp/idx/identify?",
                    },
                },
                "displaymessage": "Group Privilege granted",
                "eventtype": "group.privilege.grant",
                "legacyeventtype": "group.privilege.grant",
                "outcome": {"result": "FAILURE"},
                "published": "2022-12-13 00:58:19.811",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Springfield",
                                "country": "United States",
                                "geolocation": {"lat": 11.111, "lon": -70},
                                "postalCode": "1234",
                                "state": "California",
                            },
                            "ip": "1.2.3.4",
                            "version": "V4",
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 11351,
                    "asOrg": "charter communications inc",
                    "domain": "rr.com",
                    "isProxy": False,
                    "isp": "charter communications inc",
                },
                "severity": "WARN",
                "target": [
                    {"alternateId": "App (123)", "displayName": "App (123)", "id": "12345", "type": "AppInstance"},
                ],
                "transaction": {"detail": {}, "id": "aaa-bbb-123", "type": "WEB"},
                "uuid": "aa-11-22-33-44-bb",
                "version": "0",
            },
        ),
        RuleTest(
            name="Non Event",
            expected_result=False,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpsons",
                    "id": "00ABC123",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "xyz1234"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Springfield",
                        "country": "United States",
                        "geolocation": {"lat": 11.111, "lon": -70},
                        "postalCode": "1234",
                        "state": "California",
                    },
                    "ipAddress": "1.2.3.4",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "authnRequestId": "ABC123",
                        "deviceFingerprint": "009988771ABC",
                        "dtHash": "123abc1234",
                        "requestId": "abc-111-adf",
                        "requestUri": "/idp/idx/identify",
                        "threatSuspected": "false",
                        "url": "/idp/idx/identify?",
                    },
                },
                "displaymessage": "User attempted to reuse tokens",
                "eventtype": "app.token.reuse",
                "legacyeventtype": "app.token.reuse",
                "outcome": {"result": "FAILURE"},
                "published": "2022-12-13 00:58:19.811",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Springfield",
                                "country": "United States",
                                "geolocation": {"lat": 11.111, "lon": -70},
                                "postalCode": "1234",
                                "state": "California",
                            },
                            "ip": "1.2.3.4",
                            "version": "V4",
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 11351,
                    "asOrg": "charter communications inc",
                    "domain": "rr.com",
                    "isProxy": False,
                    "isp": "charter communications inc",
                },
                "severity": "WARN",
                "target": [
                    {"alternateId": "App (123)", "displayName": "App (123)", "id": "12345", "type": "AppInstance"},
                ],
                "transaction": {"detail": {}, "id": "aaa-bbb-123", "type": "WEB"},
                "uuid": "aa-11-22-33-44-bb",
                "version": "0",
            },
        ),
    ]



from ipaddress import ip_address

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import eks_panther_obj_ref
from pypanther.helpers.ipinfo import get_ipinfo_asn
from pypanther.helpers.misp import get_misp_warning_lists


@panther_managed
class AmazonEKSAuditSystemNamespaceFromPublicIP(Rule):
    id = "Amazon.EKS.Audit.SystemNamespaceFromPublicIP-prototype"
    display_name = "EKS Audit Log Reporting system Namespace is Used From A Public IP"
    log_types = [LogType.AMAZON_EKS_AUDIT]
    tags = [
        "EKS",
        "Initial Access",
        "Lateral Movement",
        "Exploit Public-Facing Application",
        "Remote Services",
        "Cloud Services",
    ]
    reports = {"MITRE ATT&CK": ["TA0001:T1190", "TA0008:T1201.007"]}
    default_reference = "https://docs.aws.amazon.com/eks/latest/userguide/network_reqs.html"
    default_severity = Severity.INFO
    create_alert = False
    default_description = 'This detection identifies if an activity is recorded in the Kubernetes audit log where the user:username attribute begins with "system:" or "eks:" and the requests originating IP Address is a Public IP Address\n'
    dedup_period_minutes = 1440
    summary_attributes = ["user:username", "p_source_label"]
    # Explicitly ignore eks:node-manager and eks:addon-manager
    #  which are run as Lambdas and originate from public IPs
    AMZ_PUBLICS = {"eks:addon-manager", "eks:node-manager"}
    # Alert if
    #   the username starts ( with system: or eks: )
    #   and
    #   sourceIPs[0] is a Public Address
    #   but exclude legitimate EKS nodes from AWS infrastructure IPs
    # If not defined, defaults to the rule display name or rule ID.

    def is_aws_infrastructure_ip(self, event):
        """Check if the source IP is from AWS infrastructure based on enrichment data."""
        p_eks = eks_panther_obj_ref(event)
        source_ip = p_eks.get("sourceIPs", [None])[0]
        if not source_ip:
            return False
        # Check MISP warning lists for AWS IP ranges
        misp_data = get_misp_warning_lists(event)
        if misp_data and misp_data.has_warning_list_id(source_ip, "amazon-aws"):
            return True
        # Check ipinfo ASN for Amazon using helper class
        ipinfo_asn_data = get_ipinfo_asn(event)
        if ipinfo_asn_data:
            asn_value = ipinfo_asn_data.asn("sourceIPs")[0]
            domain_value = ipinfo_asn_data.domain("sourceIPs")[0]
            if asn_value == "AS16509" and domain_value == "amazon.com":
                return True
        return False

    def is_legitimate_eks_node(self, event):
        """Check if this is a legitimate EKS node based on username and user groups."""
        p_eks = eks_panther_obj_ref(event)
        actor = p_eks.get("actor", "")
        # Check if it's a system node
        if actor.startswith("system:node:"):
            user_groups = event.deep_get("user", "groups", default=[])
            # Legitimate EKS nodes should be in system:nodes and system:authenticated groups
            return "system:nodes" in user_groups and "system:authenticated" in user_groups
        return False

    def rule(self, event):
        if event.get("stage", "") != "ResponseComplete":
            return False
        # We explicitly ignore 403 here. There is another
        #  detection that monitors for 403 volume-by-originating-ip
        if event.get("responseStatus", {}).get("code", 0) == 403:
            return False
        p_eks = eks_panther_obj_ref(event)
        # Ignore AWS managed services (addon-manager, node-manager)
        if (
            p_eks.get("actor") in self.AMZ_PUBLICS
            and ":assumed-role/AWSWesleyClusterManagerLambda"
            in event.deep_get("user", "extra", "arn", default=["not found"])[0]
        ):
            return False
        if self.is_legitimate_eks_node(event) and self.is_aws_infrastructure_ip(event):
            return False
        # Check if this is a system or EKS user from a public IP
        actor = p_eks.get("actor", "")
        if (actor.startswith("system:") or actor.startswith("eks:")) and ip_address(
            p_eks.get("sourceIPs")[0],
        ).is_global:
            return True
        return False

    def title(self, event):
        p_eks = eks_panther_obj_ref(event)
        return f"[{p_eks.get('actor')}] executed [{p_eks.get('verb')}] for resource [{p_eks.get('resource')}] in ns [{p_eks.get('ns')}] on [{p_eks.get('p_source_label')}] from [{p_eks.get('sourceIPs')[0]}]"

    def dedup(self, event):
        p_eks = eks_panther_obj_ref(event)
        return f"{p_eks.get('p_source_label')}_eks_system_namespace_{p_eks.get('sourceIPs')[0]}"

    def alert_context(self, event):
        p_eks = eks_panther_obj_ref(event)
        mutable_event = event.to_dict()
        mutable_event["p_eks"] = p_eks
        return dict(mutable_event)

    tests = [
        RuleTest(
            name="non-system username",
            expected_result=False,
            log={
                "annotations": {"authorization.k8s.io/decision": "allow", "authorization.k8s.io/reason": ""},
                "apiVersion": "audit.k8s.io/v1",
                "auditID": "35506555-dffc-4337-b2b1-c4af52b88e18",
                "kind": "Event",
                "level": "Request",
                "objectRef": {
                    "apiVersion": "v1",
                    "name": "kube-bench-drn4j",
                    "namespace": "default",
                    "resource": "pods",
                    "subresource": "log",
                },
                "p_any_aws_account_ids": ["123412341234"],
                "p_any_aws_arns": [
                    "arn:aws:iam::123412341234:role/DevAdministrator",
                    "arn:aws:sts::123412341234:assumed-role/DevAdministrator/1669660343296132000",
                ],
                "p_any_ip_addresses": ["5.5.5.5"],
                "p_any_usernames": ["kubernetes-admin"],
                "p_event_time": "2022-11-29 00:09:04.38",
                "p_log_type": "Amazon.EKS.Audit",
                "p_parse_time": "2022-11-29 00:10:25.067",
                "p_row_id": "2e4ab474b0f0f7a4a8fff4f014a9b32a",
                "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
                "p_source_label": "example-cluster-eks-logs",
                "requestReceivedTimestamp": "2022-11-29 00:09:04.38",
                "requestURI": "/api/v1/namespaces/default/pods/kube-bench-drn4j/log?container=kube-bench",
                "responseStatus": {"code": 200},
                "sourceIPs": ["5.5.5.5"],
                "stage": "ResponseStarted",
                "stageTimestamp": "2022-11-29 00:09:04.392",
                "user": {
                    "extra": {
                        "accessKeyId": ["ASIARLIVEKVNN6Y6J5UW"],
                        "arn": ["arn:aws:sts::123412341234:assumed-role/DevAdministrator/1669660343296132000"],
                        "canonicalArn": ["arn:aws:iam::123412341234:role/DevAdministrator"],
                        "sessionName": ["1669660343296132000"],
                    },
                    "groups": ["system:masters", "system:authenticated"],
                    "uid": "aws-iam-authenticator:123412341234:AROARLIVEKVNIRVGDLJWJ",
                    "username": "kubernetes-admin",
                },
                "userAgent": "kubectl/v1.25.4 (darwin/arm64) kubernetes/872a965",
                "verb": "get",
            },
        ),
        RuleTest(
            name="system username - private ip",
            expected_result=False,
            log={
                "annotations": {
                    "authorization.k8s.io/decision": "allow",
                    "authorization.k8s.io/reason": 'RBAC: allowed by ClusterRoleBinding "system:coredns" of ClusterRole "system:coredns" to ServiceAccount "coredns/kube-system"',
                },
                "apiVersion": "audit.k8s.io/v1",
                "auditID": "e2626946-90e1-4d0c-829e-ad5a78572926",
                "kind": "Event",
                "level": "Metadata",
                "objectRef": {"apiGroup": "discovery.k8s.io", "apiVersion": "v1", "resource": "endpointslices"},
                "p_any_ip_addresses": ["10.0.27.115"],
                "p_any_usernames": ["system:serviceaccount:kube-system:coredns"],
                "p_event_time": "2022-11-29 22:34:06.892",
                "p_log_type": "Amazon.EKS.Audit",
                "p_parse_time": "2022-11-29 22:45:25.024",
                "p_row_id": "c2a7d8dd7c858dcae0a1aaf314b2a207",
                "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
                "p_source_label": "example-cluster-eks-logs",
                "requestReceivedTimestamp": "2022-11-29 22:34:06.892",
                "requestURI": "/apis/discovery.k8s.io/v1/endpointslices?allowWatchBookmarks=true&resourceVersion=2528212&timeout=5m56s&timeoutSeconds=356&watch=true",
                "responseStatus": {"code": 200},
                "sourceIPs": ["10.0.27.115"],
                "stage": "ResponseComplete",
                "stageTimestamp": "2022-11-29 22:40:02.903",
                "user": {
                    "extra": {
                        "authentication_kubernetes_io_slash_pod-name": ["coredns-57ff979f67-bl27n"],
                        "authentication_kubernetes_io_slash_pod-uid": ["5b9488ae-5563-42aa-850b-b0d82edb3e22"],
                    },
                    "groups": ["system:serviceaccounts", "system:serviceaccounts:kube-system", "system:authenticated"],
                    "uid": "5e4461f9-f529-4e66-9343-0b0cc9452284",
                    "username": "system:serviceaccount:kube-system:coredns",
                },
                "userAgent": "Go-http-client/2.0",
                "verb": "watch",
            },
        ),
        RuleTest(
            name="403 from Public IP zero count",
            expected_result=True,
            log={
                "annotations": {
                    "authorization.k8s.io/decision": "allow",
                    "authorization.k8s.io/reason": 'RBAC: allowed by ClusterRoleBinding "system:coredns" of ClusterRole "system:coredns" to ServiceAccount "coredns/kube-system"',
                },
                "apiVersion": "audit.k8s.io/v1",
                "auditID": "e2626946-90e1-4d0c-829e-ad5a78572926",
                "kind": "Event",
                "level": "Metadata",
                "objectRef": {"apiGroup": "discovery.k8s.io", "apiVersion": "v1", "resource": "endpointslices"},
                "p_any_ip_addresses": ["5.5.5.5"],
                "p_any_usernames": ["system:serviceaccount:kube-system:coredns"],
                "p_event_time": "2022-11-29 22:34:06.892",
                "p_log_type": "Amazon.EKS.Audit",
                "p_parse_time": "2022-11-29 22:45:25.024",
                "p_row_id": "c2a7d8dd7c858dcae0a1aaf314b2a207",
                "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
                "p_source_label": "example-cluster-eks-logs",
                "requestReceivedTimestamp": "2022-11-29 22:34:06.892",
                "requestURI": "/apis/discovery.k8s.io/v1/endpointslices?allowWatchBookmarks=true&resourceVersion=2528212&timeout=5m56s&timeoutSeconds=356&watch=true",
                "responseStatus": {"code": 200},
                "sourceIPs": ["5.5.5.5"],
                "stage": "ResponseComplete",
                "stageTimestamp": "2022-11-29 22:40:02.903",
                "user": {
                    "extra": {
                        "authentication_kubernetes_io_slash_pod-name": ["coredns-57ff979f67-bl27n"],
                        "authentication_kubernetes_io_slash_pod-uid": ["5b9488ae-5563-42aa-850b-b0d82edb3e22"],
                    },
                    "groups": ["system:serviceaccounts", "system:serviceaccounts:kube-system", "system:authenticated"],
                    "uid": "5e4461f9-f529-4e66-9343-0b0cc9452284",
                    "username": "system:serviceaccount:kube-system:coredns",
                },
                "userAgent": "Go-http-client/2.0",
                "verb": "watch",
            },
        ),
        RuleTest(
            name="system username - public ip - not ResponseComplete",
            expected_result=False,
            log={
                "annotations": {
                    "authorization.k8s.io/decision": "allow",
                    "authorization.k8s.io/reason": 'RBAC: allowed by ClusterRoleBinding "system:coredns" of ClusterRole "system:coredns" to ServiceAccount "coredns/kube-system"',
                },
                "apiVersion": "audit.k8s.io/v1",
                "auditID": "c8c5bc49-cd5d-45d6-999c-b55783c7840f",
                "kind": "Event",
                "level": "Metadata",
                "objectRef": {"apiGroup": "discovery.k8s.io", "apiVersion": "v1", "resource": "endpointslices"},
                "p_any_ip_addresses": ["5.5.5.5"],
                "p_any_usernames": ["system:serviceaccount:kube-system:coredns"],
                "p_event_time": "2022-11-29 22:46:37.995",
                "p_log_type": "Amazon.EKS.Audit",
                "p_parse_time": "2022-11-29 22:50:24.942",
                "p_row_id": "fa229ed1d0b18094f4a1aff3149531",
                "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
                "p_source_label": "example-cluster-eks-logs",
                "requestReceivedTimestamp": "2022-11-29 22:46:37.995",
                "requestURI": "/apis/discovery.k8s.io/v1/endpointslices?allowWatchBookmarks=true&resourceVersion=2529923&timeout=6m41s&timeoutSeconds=401&watch=true",
                "responseStatus": {"code": 200},
                "sourceIPs": ["5.5.5.5"],
                "stage": "ResponseStarted",
                "stageTimestamp": "2022-11-29 22:46:38.013",
                "user": {
                    "extra": {
                        "authentication_kubernetes_io_slash_pod-name": ["coredns-57ff979f67-bl27n"],
                        "authentication_kubernetes_io_slash_pod-uid": ["5b9488ae-5563-42aa-850b-b0d82edb3e22"],
                    },
                    "groups": ["system:serviceaccounts", "system:serviceaccounts:kube-system", "system:authenticated"],
                    "uid": "5e4461f9-f529-4e66-9343-0b0cc9452284",
                    "username": "system:serviceaccount:kube-system:coredns",
                },
                "userAgent": "Go-http-client/2.0",
                "verb": "watch",
            },
        ),
        RuleTest(
            name="system username - public ip - 403",
            expected_result=False,
            log={
                "annotations": {
                    "authorization.k8s.io/decision": "allow",
                    "authorization.k8s.io/reason": 'RBAC: allowed by ClusterRoleBinding "system:coredns" of ClusterRole "system:coredns" to ServiceAccount "coredns/kube-system"',
                },
                "apiVersion": "audit.k8s.io/v1",
                "auditID": "c8c5bc49-cd5d-45d6-999c-b55783c7840f",
                "kind": "Event",
                "level": "Metadata",
                "objectRef": {"apiGroup": "discovery.k8s.io", "apiVersion": "v1", "resource": "endpointslices"},
                "p_any_ip_addresses": ["5.5.5.5"],
                "p_any_usernames": ["system:serviceaccount:kube-system:coredns"],
                "p_event_time": "2022-11-29 22:46:37.995",
                "p_log_type": "Amazon.EKS.Audit",
                "p_parse_time": "2022-11-29 22:50:24.942",
                "p_row_id": "fa229ed1d0b18094f4a1aff3149531",
                "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
                "p_source_label": "example-cluster-eks-logs",
                "requestReceivedTimestamp": "2022-11-29 22:46:37.995",
                "requestURI": "/apis/discovery.k8s.io/v1/endpointslices?allowWatchBookmarks=true&resourceVersion=2529923&timeout=6m41s&timeoutSeconds=401&watch=true",
                "responseStatus": {"code": 403},
                "sourceIPs": ["5.5.5.5"],
                "stage": "ResponseComplete",
                "stageTimestamp": "2022-11-29 22:46:38.013",
                "user": {
                    "extra": {
                        "authentication_kubernetes_io_slash_pod-name": ["coredns-57ff979f67-bl27n"],
                        "authentication_kubernetes_io_slash_pod-uid": ["5b9488ae-5563-42aa-850b-b0d82edb3e22"],
                    },
                    "groups": ["system:serviceaccounts", "system:serviceaccounts:kube-system", "system:authenticated"],
                    "uid": "5e4461f9-f529-4e66-9343-0b0cc9452284",
                    "username": "system:serviceaccount:kube-system:coredns",
                },
                "userAgent": "Go-http-client/2.0",
                "verb": "watch",
            },
        ),
        RuleTest(
            name="eks:addon-manager from public ip as lambda",
            expected_result=False,
            log={
                "annotations": {
                    "authorization.k8s.io/decision": "allow",
                    "authorization.k8s.io/reason": 'RBAC: allowed by RoleBinding "eks:addon-manager/kube-system" of Role "eks:addon-manager" to User "eks:addon-manager"',
                },
                "apiVersion": "audit.k8s.io/v1",
                "auditID": "43410f6e-9c19-482b-b2c7-f2cde260b0e9",
                "kind": "Event",
                "level": "Request",
                "objectRef": {
                    "apiGroup": "apps",
                    "apiVersion": "v1",
                    "name": "coredns",
                    "namespace": "kube-system",
                    "resource": "deployments",
                },
                "p_any_aws_account_ids": ["123412341234"],
                "p_any_aws_arns": [
                    "arn:aws:iam::123412341234:role/AWSWesleyClusterManagerLambda-Add-AddonManagerRole-G332QAM69HWF",
                    "arn:aws:sts::123412341234:assumed-role/AWSWesleyClusterManagerLambda-Add-AddonManagerRole-G332QAM69HWF/1669918824986835422",
                ],
                "p_any_ip_addresses": ["35.163.244.48"],
                "p_any_usernames": ["eks:addon-manager"],
                "p_event_time": "2022-12-01 18:20:25.054",
                "p_log_type": "Amazon.EKS.Audit",
                "p_parse_time": "2022-12-01 18:24:24.734",
                "p_row_id": "a22a2e182591cfb8ead2f7f7149215",
                "p_source_id": "4c859cd4-9406-469b-9e0e-c2dc1bee24fa",
                "p_source_label": "example-cluster-eks-logs",
                "requestReceivedTimestamp": "2022-12-01 18:20:25.054",
                "requestURI": "/apis/apps/v1/namespaces/kube-system/deployments/coredns?timeout=10s",
                "responseStatus": {"code": 200},
                "sourceIPs": ["35.163.244.48"],
                "stage": "ResponseComplete",
                "stageTimestamp": "2022-12-01 18:20:25.078",
                "user": {
                    "extra": {
                        "accessKeyId": ["ASIAXXXXXXXXXXXXXXXX"],
                        "arn": [
                            "arn:aws:sts::123412341234:assumed-role/AWSWesleyClusterManagerLambda-Add-AddonManagerRole-G332QAM69HWF/1669918824986835422",
                        ],
                        "canonicalArn": [
                            "arn:aws:iam::123412341234:role/AWSWesleyClusterManagerLambda-Add-AddonManagerRole-G332QAM69HWF",
                        ],
                        "sessionName": ["1669918824986835422"],
                    },
                    "groups": ["system:authenticated"],
                    "uid": "aws-iam-authenticator:123412341234:AROATAVZDPHFJWUSNL3ZV",
                    "username": "eks:addon-manager",
                },
                "userAgent": "Go-http-client/1.1",
                "verb": "get",
            },
        ),
        RuleTest(
            name="legitimate EKS node from AWS IP - should not alert",
            expected_result=False,
            log={
                "annotations": {
                    "authentication.kubernetes.io/issued-credential-id": "JTI=2e6389af-b8bc-4c26-92d2-714fbe9cb4fa",
                    "authorization.k8s.io/decision": "allow",
                    "authorization.k8s.io/reason": "",
                },
                "apiVersion": "audit.k8s.io/v1",
                "auditID": "e5ca89e7-8686-4b7c-b666-f8b863b6f21c",
                "kind": "Event",
                "level": "Metadata",
                "objectRef": {
                    "apiVersion": "v1",
                    "name": "kube-proxy",
                    "namespace": "kube-system",
                    "resource": "serviceaccounts",
                    "subresource": "token",
                },
                "p_any_ip_addresses": ["54.212.83.236"],
                "p_enrichment": {
                    "MISP Warning Lists": {
                        "p_any_ip_addresses": [
                            {
                                "cidr": "54.208.0.0/13",
                                "p_match": "54.212.83.236",
                                "warning_lists": [
                                    {
                                        "description": "Amazon AWS IP address ranges (https://ip-ranges.amazonaws.com/ip-ranges.json)",
                                        "id": "amazon-aws",
                                        "name": "List of known Amazon AWS IP address ranges",
                                        "version": 20250719,
                                    },
                                ],
                            },
                        ],
                    },
                    "ipinfo_asn": {
                        "sourceIPs": [
                            {
                                "asn": "AS16509",
                                "domain": "amazon.com",
                                "name": "Amazon.com, Inc.",
                                "p_match": "54.212.83.236",
                                "route": "54.212.0.0/16",
                                "type": "hosting",
                            },
                        ],
                    },
                },
                "p_event_time": "2025-08-26 17:40:49.320496000",
                "p_log_type": "Amazon.EKS.Audit",
                "requestReceivedTimestamp": "2025-08-26 17:40:49.320496000",
                "requestURI": "/api/v1/namespaces/kube-system/serviceaccounts/kube-proxy/token",
                "responseStatus": {"code": 201},
                "sourceIPs": ["54.212.83.236"],
                "stage": "ResponseComplete",
                "stageTimestamp": "2025-08-26 17:40:49.330617000",
                "user": {
                    "groups": ["system:nodes", "system:authenticated"],
                    "username": "system:node:ip-192-168-3-178.us-west-2.compute.internal",
                },
                "userAgent": "kubelet/v1.32.0 (linux/arm64) kubernetes/e105b10",
                "verb": "create",
            },
        ),
        RuleTest(
            name="system user from non-AWS public IP - should alert",
            expected_result=True,
            log={
                "annotations": {"authorization.k8s.io/decision": "allow", "authorization.k8s.io/reason": ""},
                "apiVersion": "audit.k8s.io/v1",
                "auditID": "e2626946-90e1-4d0c-829e-ad5a78572926",
                "kind": "Event",
                "level": "Metadata",
                "objectRef": {
                    "apiVersion": "v1",
                    "name": "kube-proxy",
                    "namespace": "kube-system",
                    "resource": "serviceaccounts",
                    "subresource": "token",
                },
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_enrichment": {
                    "ipinfo_asn": {
                        "sourceIPs": [
                            {
                                "asn": "AS12345",
                                "domain": "example-isp.com",
                                "name": "Example ISP",
                                "p_match": "1.2.3.4",
                                "route": "1.2.3.0/24",
                                "type": "isp",
                            },
                        ],
                    },
                },
                "p_event_time": "2025-08-26 17:40:49.320496000",
                "p_log_type": "Amazon.EKS.Audit",
                "requestReceivedTimestamp": "2025-08-26 17:40:49.320496000",
                "requestURI": "/api/v1/namespaces/kube-system/serviceaccounts/kube-proxy/token",
                "responseStatus": {"code": 201},
                "sourceIPs": ["1.2.3.4"],
                "stage": "ResponseComplete",
                "stageTimestamp": "2025-08-26 17:40:49.330617000",
                "user": {"groups": ["system:nodes", "system:authenticated"], "username": "system:node:suspicious-node"},
                "userAgent": "kubelet/v1.32.0 (linux/arm64) kubernetes/e105b10",
                "verb": "create",
            },
        ),
        RuleTest(
            name="attacker from non-AWS IP targeting AWS resource - should alert despite AWS target",
            expected_result=True,
            log={
                "annotations": {"authorization.k8s.io/decision": "allow", "authorization.k8s.io/reason": ""},
                "apiVersion": "audit.k8s.io/v1",
                "auditID": "e2626946-90e1-4d0c-829e-ad5a78572926",
                "kind": "Event",
                "level": "Metadata",
                "objectRef": {
                    "apiVersion": "v1",
                    "name": "kube-proxy",
                    "namespace": "kube-system",
                    "resource": "serviceaccounts",
                    "subresource": "token",
                },
                "p_any_ip_addresses": ["1.2.3.4", "54.212.83.236"],
                "p_enrichment": {
                    "MISP Warning Lists": {
                        "p_any_ip_addresses": [
                            {
                                "cidr": "54.208.0.0/13",
                                "p_match": "54.212.83.236",
                                "warning_lists": [
                                    {
                                        "description": "Amazon AWS IP address ranges",
                                        "id": "amazon-aws",
                                        "name": "List of known Amazon AWS IP address ranges",
                                    },
                                ],
                            },
                        ],
                    },
                    "ipinfo_asn": {
                        "sourceIPs": [
                            {
                                "asn": "AS12345",
                                "domain": "example-isp.com",
                                "name": "Example ISP",
                                "p_match": "1.2.3.4",
                                "route": "1.2.3.0/24",
                                "type": "isp",
                            },
                        ],
                    },
                },
                "p_event_time": "2025-08-26 17:40:49.320496000",
                "p_log_type": "Amazon.EKS.Audit",
                "requestReceivedTimestamp": "2025-08-26 17:40:49.320496000",
                "requestURI": "/api/v1/namespaces/kube-system/serviceaccounts/kube-proxy/token",
                "responseStatus": {"code": 201},
                "sourceIPs": ["1.2.3.4"],
                "stage": "ResponseComplete",
                "stageTimestamp": "2025-08-26 17:40:49.330617000",
                "user": {"groups": ["system:nodes", "system:authenticated"], "username": "system:node:suspicious-node"},
                "userAgent": "kubelet/v1.32.0 (linux/arm64) kubernetes/e105b10",
                "verb": "create",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp import gcp_alert_context


@panther_managed
class GCPTagBindingCreation(Rule):
    id = "GCP.Tag.Binding.Creation-prototype"
    default_description = "Detects the creation of tag bindings in GCP, which could be part of a privilege escalation attempt using tag-based access control.\n"
    display_name = "GCP Tag Binding Creation"
    log_types = [LogType.GCP_AUDIT_LOG]
    create_alert = False
    default_runbook = "Verify if the user has legitimate business need for creating this tag binding. If unauthorized, revoke the tag binding and review IAM policies.\n"
    default_severity = Severity.INFO
    tags = ["attack.privilege_escalation", "attack.t1548", "gcp", "iam", "tagbinding"]

    def rule(self, event):
        method_name = event.deep_get("protoPayload", "methodName", default="")
        return method_name.endswith("TagBindings.CreateTagBinding")

    def title(self, event):
        principal = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<UNKNOWN>")
        resource = event.deep_get("protoPayload", "resourceName", default="<UNKNOWN>")
        return f"GCP Tag Binding Creation by {principal} - {resource}"

    def alert_context(self, event):
        return gcp_alert_context(event)

    tests = [
        RuleTest(
            name="Tag Binding Creation",
            expected_result=True,
            log={
                "protoPayload": {
                    "methodName": "TagBindings.CreateTagBinding",
                    "authenticationInfo": {"principalEmail": "test@example.com"},
                    "resourceName": "projects/test-project",
                },
                "resource": {"labels": {"project_id": "test-project"}},
                "timestamp": "2024-01-01T00:00:00Z",
            },
        ),
        RuleTest(
            name="Normal Operation",
            expected_result=False,
            log={
                "protoPayload": {
                    "methodName": "compute.instances.list",
                    "authenticationInfo": {"principalEmail": "test@example.com"},
                    "resourceName": "projects/test-project",
                },
                "resource": {"labels": {"project_id": "test-project"}},
                "timestamp": "2024-01-01T00:00:00Z",
            },
        ),
    ]



from pypanther.rules.aws_bedrockmodelinvocation.aws_bedrockmodelinvocation_abnormaltokenusage import (
    AWSBedrockModelInvocationAbnormalTokenUsage as AWSBedrockModelInvocationAbnormalTokenUsage,
)
from pypanther.rules.aws_bedrockmodelinvocation.aws_bedrockmodelinvocation_guardrailintervened import (
    AWSBedrockModelInvocationGuardRailIntervened as AWSBedrockModelInvocationGuardRailIntervened,
)



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.okta import okta_alert_context


@panther_managed
class OktaAdminRoleAssigned(Rule):
    id = "Okta.AdminRoleAssigned-prototype"
    display_name = "Okta Admin Role Assigned"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    tags = ["Identity & Access Management", "Okta", "Privilege Escalation:Valid Accounts"]
    reports = {"MITRE ATT&CK": ["TA0004:T1078"]}
    default_severity = Severity.INFO
    default_description = "A user has been granted administrative privileges in Okta"
    default_reference = "https://help.okta.com/en/prod/Content/Topics/Security/administrators-admin-comparison.htm"
    default_runbook = "Reach out to the user if needed to validate the activity"
    dedup_period_minutes = 15
    summary_attributes = ["eventType", "severity", "displayMessage", "p_any_ip_addresses"]
    ADMIN_PATTERN = re.compile("[aA]dministrator")

    def rule(self, event):
        return (
            event.get("eventType", None) == "user.account.privilege.grant"
            and event.deep_get("outcome", "result") == "SUCCESS"
            and bool(
                self.ADMIN_PATTERN.search(event.deep_get("debugContext", "debugData", "privilegeGranted", default="")),
            )
        )

    def dedup(self, event):
        return event.deep_get("debugContext", "debugData", "requestId", default="<UNKNOWN_REQUEST_ID>")

    def title(self, event):
        target = event.get("target", [{}])
        display_name = target[0].get("displayName", "MISSING DISPLAY NAME") if target else ""
        alternate_id = target[0].get("alternateId", "MISSING ALTERNATE ID") if target else ""
        privilege = event.deep_get("debugContext", "debugData", "privilegeGranted", default="<UNKNOWN_PRIVILEGE>")
        return f"{event.deep_get('actor', 'displayName')} <{event.deep_get('actor', 'alternateId')}> granted [{privilege}] privileges to {display_name} <{alternate_id}>"

    def alert_context(self, event):
        return okta_alert_context(event)

    def severity(self, event):
        if "Super administrator" in event.deep_get("debugContext", "debugData", "privilegeGranted", default=""):
            return "HIGH"
        return "INFO"

    tests = [
        RuleTest(
            name="Admin Access Assigned",
            expected_result=True,
            log={
                "uuid": "2a992f80-d1ad-4f62-900e-8c68bb72a21b",
                "published": "2020-11-25 21:27:03.496000000",
                "eventType": "user.account.privilege.grant",
                "version": "0",
                "severity": "INFO",
                "legacyEventType": "core.user.admin_privilege.granted",
                "displayMessage": "Grant user privilege",
                "actor": {
                    "id": "00uu1uuuuIlllaaaa356",
                    "type": "User",
                    "alternateId": "jack@acme.io",
                    "displayName": "Jack Naglieri",
                },
                "client": {
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36",
                    },
                    "geographicalContext": {
                        "geolocation": {"lat": 37.7852, "lon": -122.3874},
                        "city": "San Francisco",
                        "state": "California",
                        "country": "United States",
                        "postalCode": "94105",
                    },
                    "zone": "null",
                    "ipAddress": "136.24.229.58",
                    "device": "Computer",
                },
                "request": {},
                "outcome": {"result": "SUCCESS"},
                "target": [
                    {
                        "id": "00u6eup97mAJZWYmP357",
                        "type": "User",
                        "alternateId": "alice@acme.io",
                        "displayName": "Alice Green",
                    },
                ],
                "transaction": {},
                "debugContext": {
                    "debugData": {
                        "privilegeGranted": "Organization administrator, Application administrator (all)",
                        "requestUri": "/api/internal/administrators/00u6eu8c68bb72a21b57",
                        "threatSuspected": "false",
                        "url": "/api/internal/administrators/00u6eu8c68bb72a21b57",
                        "requestId": "X777JJ9sssQQHHrrrQTyYQAABBE",
                    },
                },
                "authenticationContext": {},
                "securityContext": {},
            },
        ),
        RuleTest(
            name="Super Admin Access Assigned (High sev)",
            expected_result=True,
            log={
                "uuid": "2a992f80-d1ad-4f62-900e-8c68bb72a21b",
                "published": "2020-11-25 21:27:03.496000000",
                "eventType": "user.account.privilege.grant",
                "version": "0",
                "severity": "INFO",
                "legacyEventType": "core.user.admin_privilege.granted",
                "displayMessage": "Grant user privilege",
                "actor": {
                    "id": "00uu1uuuuIlllaaaa356",
                    "type": "User",
                    "alternateId": "jack@acme.io",
                    "displayName": "Jack Naglieri",
                },
                "client": {
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36",
                    },
                    "geographicalContext": {
                        "geolocation": {"lat": 37.7852, "lon": -122.3874},
                        "city": "San Francisco",
                        "state": "California",
                        "country": "United States",
                        "postalCode": "94105",
                    },
                    "zone": "null",
                    "ipAddress": "136.24.229.58",
                    "device": "Computer",
                },
                "request": {},
                "outcome": {"result": "SUCCESS"},
                "target": [
                    {
                        "id": "00u6eup97mAJZWYmP357",
                        "type": "User",
                        "alternateId": "alice@acme.io",
                        "displayName": "Alice Green",
                    },
                ],
                "transaction": {},
                "debugContext": {
                    "debugData": {
                        "privilegeGranted": "Super administrator, Read only admin",
                        "requestUri": "/api/internal/administrators/00u6eu8c68bb72a21b57",
                        "threatSuspected": "false",
                        "url": "/api/internal/administrators/00u6eu8c68bb72a21b57",
                        "requestId": "X777JJ9sssQQHHrrrQTyYQAABBE",
                    },
                },
                "authenticationContext": {},
                "securityContext": {},
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp import gcp_alert_context


@panther_managed
class GCPFirewallRuleModified(Rule):
    dedup_period_minutes = 90
    display_name = "GCP Firewall Rule Modified"
    id = "GCP.Firewall.Rule.Modified-prototype"
    default_severity = Severity.LOW
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Firewall", "Networking", "Infrastructure"]
    default_description = "This rule detects modifications to GCP firewall rules.\n"
    default_runbook = "Ensure that the rule modification was expected. Firewall rule changes can cause service interruptions or outages.\n"
    default_reference = "https://cloud.google.com/firewall/docs/about-firewalls"
    RULE_MODIFIED_PARTS = [".Firewall.Update", ".compute.firewalls.patch", ".compute.firewalls.update"]

    def rule(self, event):
        method = event.deep_get("protoPayload", "methodName", default="")
        return any(part in method for part in self.RULE_MODIFIED_PARTS)

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        resource = event.deep_get("protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        return f"[GCP]: [{actor}] modified firewall rule on [{resource}]"

    def dedup(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        return actor

    def alert_context(self, event):
        return gcp_alert_context(event)

    tests = [
        RuleTest(
            name="compute.firewalls.update-should-alert",
            expected_result=True,
            log={
                "insertid": "-xxxxxxxxxxxx",
                "logname": "projects/test-project-123456/cloudaudit.googleapis.com%2Factivity",
                "operation": {
                    "first": True,
                    "id": "operation-1684869580331-5fc6144d418a9-e1332ca3-59c615ac",
                    "producer": "compute.googleapis.com",
                },
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@domain.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "compute.firewalls.update",
                            "resourceAttributes": {
                                "name": "projects/test-project-123456/global/firewalls/firewall-create",
                                "service": "compute",
                                "type": "compute.firewalls",
                            },
                        },
                        {
                            "granted": True,
                            "permission": "compute.networks.updatePolicy",
                            "resourceAttributes": {
                                "name": "projects/test-project-123456/global/networks/default",
                                "service": "compute",
                                "type": "compute.networks",
                            },
                        },
                    ],
                    "methodName": "v1.compute.firewalls.patch",
                    "request": {
                        "@type": "type.googleapis.com/compute.firewalls.patch",
                        "denieds": [{"IPProtocol": "all"}],
                    },
                    "requestMetadata": {
                        "callerIP": "12.12.12.12",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                        "destinationAttributes": {},
                        "requestAttributes": {
                            "auth": {},
                            "reason": "8uSywAYQGg5Db2xpc2V1bSBGbG93cw",
                            "time": "2023-05-23T19:19:41.154751Z",
                        },
                    },
                    "resourceName": "projects/test-project-123456/global/firewalls/firewall-create",
                    "response": {
                        "@type": "type.googleapis.com/operation",
                        "id": "896785227463044899",
                        "insertTime": "2023-05-23T12:19:40.876-07:00",
                        "name": "operation-1684869580331-5fc6144d418a9-e1332ca3-59c615ac",
                        "operationType": "patch",
                        "progress": "0",
                        "selfLink": "https://www.googleapis.com/compute/v1/projects/test-project-123456/global/operations/operation-1684869580331-5fc6144d418a9-e1332ca3-59c615ac",
                        "selfLinkWithId": "https://www.googleapis.com/compute/v1/projects/test-project-123456/global/operations/896785227463044899",
                        "startTime": "2023-05-23T12:19:40.888-07:00",
                        "status": "RUNNING",
                        "targetId": "6563507997690081088",
                        "targetLink": "https://www.googleapis.com/compute/v1/projects/test-project-123456/global/firewalls/firewall-create",
                        "user": "user@domain.com",
                    },
                    "serviceName": "compute.googleapis.com",
                },
                "receivetimestamp": "2023-05-23 19:19:41.238",
                "resource": {
                    "labels": {"firewall_rule_id": "6563507997690081088", "project_id": "test-project-123456"},
                    "type": "gce_firewall_rule",
                },
                "severity": "NOTICE",
                "timestamp": "2023-05-23 19:19:40.353",
            },
        ),
        RuleTest(
            name="appengine.firewall.update-should-alert",
            expected_result=True,
            log={
                "insertid": "-xxxxxxxxxxxx",
                "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
                "protopayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@domain.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "appengine.applications.update",
                            "resource": "apps/test-project-123456/firewall/ingressRules/1000",
                            "resourceAttributes": {},
                        },
                    ],
                    "methodName": "google.appengine.v1.Firewall.UpdateIngressRule",
                    "requestMetadata": {
                        "callerIP": "12.12.12.12",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2023-05-23T19:28:44.663413Z"},
                    },
                    "resourceName": "apps/test-project-123456/firewall/ingressRules/1000",
                    "serviceData": {"@type": "type.googleapis.com/google.appengine.v1beta4.AuditData"},
                    "serviceName": "appengine.googleapis.com",
                    "status": {},
                },
                "receivetimestamp": "2023-05-23 19:28:45.473",
                "resource": {
                    "labels": {"module_id": "", "project_id": "test-project-123456", "version_id": "", "zone": ""},
                    "type": "gae_app",
                },
                "severity": "NOTICE",
                "timestamp": "2023-05-23 19:28:44.562",
            },
        ),
        RuleTest(
            name="compute.non-update.firewall.method-should-not-alert",
            expected_result=False,
            log={"methodName": "v1.compute.firewalls.insert"},
        ),
        RuleTest(
            name="appengine.compute.non-update.firewall.method-should-not-alert",
            expected_result=False,
            log={"methodName": "appengine.compute.v1.Firewall.PatchIngressRule"},
        ),
        RuleTest(
            name="randomservice.firewall-update.method-should-alert",
            expected_result=True,
            log={
                "protoPayload": {
                    "authenticationInfo": {"principalEmail": "user@domain.com"},
                    "methodName": "randomservice.compute.v1.Firewall.UpdateIngressRule",
                    "resourceName": "randomservice/test-project-123456/firewall/ingressRules/1000",
                    "requestMetadata": {
                        "callerIP": "12.12.12.12",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2023-05-23T19:28:44.663413Z"},
                    },
                },
                "resource": {
                    "labels": {"firewall_rule_id": "6563507997690081088", "project_id": "test-project-123456"},
                    "type": "gce_firewall_rule",
                },
            },
        ),
    ]
